###
# is_enabled_complianceMode: Checks the compliance mode
###
#' Checks if the compliance mode is enabled
#'
#' @return Returns TRUE is compliance mode is enabled. FALSE otherwise
#' @export
is_enabled_complianceMode <- function(){
  # Load
  loadSetupOptions_IQReportAPI()
  # Return
  return(.COMPLIANCE_MODE)
}


###
# IQRinitCompliance: Initializes the compliance mode by setting
#  the COMPLIANCE_MODE_SCRIPT_NAME in the global environment
###
#' Initializes the compliance mode
#'
#' The compliance mode log file generation requires the global
#' variable COMPLIANCE_MODE_SCRIPT_NAME to be defined in the global
#' environment. The user needs to take care that this is the same as
#' the script file name from his/her analysis script, generating the outputs
#' of interest for compliance tracking.
#'
#' @param scriptname Name of analysis script, generating the outputs
#'   of interest for compliance tracking.
#' @export
IQRinitCompliance <- function(scriptname){
  e__ <- globalenv()
  e__$COMPLIANCE_MODE_SCRIPT_NAME <- scriptname
}


###
# IQRoutputCSV: wrapper for utils::write.csv, allowing to handle compliance information
###
#' Exporting a data.frame as CSV file
#'
#' The compliance log file will only be generated if .COMPLIANCE_MODE is set to TRUE.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' @param data The data.frame or matrix to export
#' @param filename Character string with the filename (can include path) for export.
#'   Should include extension to the filename.
#' @param na Character to represent NA values
#' @param quote TRUE: use "" quotes, FALSE: do not (for NONMEM and MONOLIX)
#' @param row.names TRUE: exposrt row names FALSE: do not (typical use in IQRtools)
#' @param FLAGattributes FALSE: do not export custom attributes as .atr file. TRUE: do (default)
#' @export
IQRoutputCSV <- function(data,filename,na=".",quote=FALSE,row.names=FALSE,FLAGattributes=TRUE) {
  if (!is.null(filename)) {
    # Ensure folder exists
    aux_mkdir(aux_fileparts(filename)$pathname)
    # Export
    utils::write.csv(x=data,file=paste0(aux_strrep(filename,".csv",""),".csv"),na=na,quote=quote,row.names=row.names)
    # Generate logfile if needed
    genComplianceLog(paste0(aux_strrep(filename,".csv",""),".csv"))

    if (FLAGattributes) {
      # Handle potential attributes
      attrStandard__ <- c("names","row.names","class")

      attr__ <- attributes(data)
      for (k in seq_along(attrStandard__)) {
        attr__[[attrStandard__[[k]]]] <- NULL
      }

      if (length(attr__) == 0) return(invisible(NULL))

      ATTRTEXT__ <- paste0("# Attributes file for dataset ",filename,"\n\n")
      ATTRTEXT__ <- paste0(ATTRTEXT__,"atrcontents <- list()\n\n")
      for (k in seq_along(attr__)) {
        n__ <- names(attr__[k])
        v__ <- attr__[[k]]
        ATTRTEXT__ <- paste0(ATTRTEXT__,"atrcontents$",n__," <- ",paste0(deparse(v__),collapse=""),"\n\n")
      }

      # Save .atr file
      filenameATR <- paste0(aux_strrep(filename,'.csv',''),'.atr')
      aux_filewrite(ATTRTEXT__,filenameATR)

      # Generate logfile if needed
      genComplianceLog(filenameATR)
    }

  } else {
    warning("filename is NULL - no file written")
  }
}


###
# IQRoutputFile: wrapper for aux_filewrite, allowing to handle compliance information
###
#' Exporting a text file with added compliance information log file
#'
#' The compliance log file will only be generated if .COMPLIANCE_MODE is set to TRUE.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' @param text Character string to export in a file
#' @param filename Character string with the filename (can include path) for export.
#'   Should include extension to the filename.
#' @export
IQRoutputFile <- function(text,filename) {
  if (is.null(filename)) return()
  # Ensure folder exists
  aux_mkdir(aux_fileparts(filename)$pathname)
  # Write out the file
  aux_filewrite(text,filename)
  # Generate logfile if needed
  genComplianceLog(filename)
}


###
# IQRoutputRDS:
###
#' Exporting an R object to RDS format with added compliance information log file
#'
#' The compliance log file will only be generated if .COMPLIANCE_MODE is set to TRUE.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' @param object R object to export
#' @param filename Character string with the filename (can include path) for export.
#'   ".rds" extension will be added and ignored in provided filename.
#' @export
IQRoutputRDS <- function(object,filename) {
  if (is.null(filename)) return()
  filename <- paste0(aux_strrep(filename,".rds",""),".rds")
  # Ensure folder exists
  aux_mkdir(aux_fileparts(filename)$pathname)
  # Write out the RDS file
  saveRDS(object,filename)
  # Generate logfile if needed
  genComplianceLog(filename)
}


###
# IQRoutputPNG: wrapper for png, allowing to handle compliance information
###
#' Printing a single ggplot object to png file with added compliance information log file
#'
#' The compliance log file will only be generated if .COMPLIANCE_MODE is set to TRUE.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' @param gr ggplot graph object to be printed to file
#' @param filename Character string with the filename (can include path) for export.
#'   Should include extension to the filename.
#' @param res Resolution in DPI
#' @param height height of figure in inch
#' @param width width of figure in inch
#' @export
IQRoutputPNG <- function(gr, filename=NULL, res = 300, width = 21/2.54, height = 21/2.54*3/4) {
  if (!is.null(filename)) {
    aux_closePNGs()

    # Make sure we have ending .png
    filename <- paste0(aux_strrep(filename,".png",""),".png")

    # Ensure folder exists
    aux_mkdir(aux_fileparts(filename)$pathname)

    # Generate png file (pixels chosen to have 300ppi on A5 paper)
    grDevices::png(filename, height = height, width = width, bg = "transparent", res = res, units = "in")
    if ("gtable" %in% class(gr))
    {
      print(graphics::plot(gr))
    } else {
      print(gr)
    }
    # Generate logfile if needed
    genComplianceLog(filename)
    aux_closePNGs()
  }
}


###
# IQRoutputPDF: wrapper for PDF, allowing to handle compliance information
###
#' Printing a single ggplot object to a PDF file with added compliance information log file
#'
#' The compliance log file will only be generated if .COMPLIANCE_MODE is set to TRUE.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' @param gr ggplot graph object to be printed to file
#' @param filename Character string with the filename (can include path) for export.
#'   Should include extension to the filename (.pdf).
#' @param width width of figure in inch
#' @param height height of figure in inch
#' @param ... Additional arguments passed to \code{\link[grDevices]{pdf}()}
#' @export
IQRoutputPDF <- function(gr, filename=NULL, width = 21/2.54, height = 21/2.54*3/4, ...) {
  if (!is.null(filename)) {
    aux_closePDFs()

    # Make sure we have ending .pdf
    filename <- paste0(aux_strrep(filename,".pdf",""),".pdf")

    # Ensure folder exists
    aux_mkdir(aux_fileparts(filename)$pathname)

    # Generate PDF file
    grDevices::pdf(filename, height = height, width = width, ...)
    if ("gtable" %in% class(gr))
    {
      print(graphics::plot(gr))
    } else {
      print(gr)
    }
    aux_closePDFs()
    # Generate logfile if needed
    genComplianceLog(filename)
  }
}


###
# IQRoutputPDFstart: Starts output into PDF and creates the
# folder if needed
###
#' Start export to PDF
#'
#' PDF export is started with IQRoutputPDFstart. Then all graphical output
#' goes to this PDF file. PDF export into this file is ended by
#' IQRoutputPDFend. If the compliance mode is on (.COMPLIANCE_MODE is set to TRUE)
#' then the exported PDF file is annotated by an additional log file.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' It is the call to IQRoutputPDFend that actually generates the log file.
#'
#' @param filename Name of the PDF file to generate. Can include path. If
#'   the folder in the path is not present, it will be created.
#' @param width width of figure in inch
#' @param height height of figure in inch
#' @param ... Additional arguments passed to \code{\link[grDevices]{pdf}()}
#' @export
IQRoutputPDFstart <- function(filename, width = 21/2.54, height = 21/2.54*3/4, ...) {
  # Close all PDFs here ...
  aux_closePDFs()

  if (!is.null(filename)) {
    # Ensure folder exists
    aux_mkdir(aux_fileparts(filename)$pathname)
    # Remove file if it exists
    if (file.exists(filename)) {
      unlink(filename)
    }
    # Open PDF
    dummy__ <- grDevices::pdf(file = paste0(aux_strrep(filename,".pdf",""),".pdf"), onefile=TRUE, width=width, height=height, ...)
  }
}


###
# IQRoutputPDFend: Ends output into PDF and creates the
# folder if needed
###
#' Ends export to PDF
#'
#' PDF export is started with IQRoutputPDFstart. Then all graphical output
#' goes to this PDF file. PDF export into this file is ended by
#' IQRoutputPDFend. If the compliance mode is on (.COMPLIANCE_MODE is set to TRUE)
#' then the exported PDF file is annotated by an additional log file.
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The extension for the exported text file will be kept
#' unchanged in the exported log file name. In order for the log file generation to
#' work the global variable "COMPLIANCE_MODE_SCRIPT_NAME" needs to be defined by the user
#' and in each R script set to the name of the script.
#'
#' It is the call to IQRoutputPDFend that actually generates the log file.
#'
#' @param filename Name of the PDF file
#' @export
IQRoutputPDFend <- function(filename) {
  # Close all PDFs here

  aux_closePDFs()
  if (!is.null(filename)) {
    # Generate logfile if needed
    genComplianceLog(paste0(aux_strrep(filename,".pdf",""),".pdf"))
  }
}


###
# IQRoutputTable: takes a dataframe or matrix and exports
#   it to text. Can be used for reporting or other purposes.
###
#' Convert dataframe to IQRoutputTable object and allow export
#'
#' This function allows to export a data.frame as a table with optional
#' title and footer information. Export is done if a filename is provided.
#' If the compliance mode is on (.COMPLIANCE_MODE is set to TRUE) then this function
#' also generates a compliance log file (only if filename is provided).
#' The name of the generated compliance log file is the same name as the text file name
#' but with appended ".log". The function also returns an IQRoutputTable object
#' that essentially is a list with xtable, xtitle, and xfooter entries.
#'
#' @param xtable Table to be exported as dataframe
#' @param xfooter String with footer text
#' @param xtitle String with table title text
#' @param object IQRoutputTable object. If defined, xtable is not used but xfooter,
#'   xtitle and report (if defined) will be set in the object, overwriting old content
#' @param filename String with filename (can include path)
#' @param report TRUE: add reporting tags <TT>, <TR>, <TH>, <TF>
#'               FALSE: do not add (for scfreen output only)
#'               NULL: same as TRUE if xtable provided.
#' @param na.string Character string to replace NA entries by. Defaults to NULL.
#' @return IQRoutputTable object. This object can be converted to text using the
#'   function text_IQRoutputTable().
#' @export
IQRoutputTable <- function(xtable=NULL,xfooter=NULL,xtitle=NULL,object=NULL,filename=NULL,report=NULL,na.string=NULL) {

  if (!is.null(filename)) {
    filename <- paste0(aux_strrep(filename,".txt",""),".txt")
    aux_mkdir(aux_fileparts(filename)$pathname)
  }

  # Check xtable or object
  if (is.null(xtable) & is.null(object)) stop("Neither xtable nor object defined")
  if (!is.null(xtable) & !is.null(object)) stop("Both xtable and object defined")

  # Handle report default
  if (!is.null(xtable) & is.null(report)) report <- TRUE

  # Check object
  if (!is.null(object) & (!"IQRoutputTable" %in% class(object))) stop("Input argument object is not an IQRoutputTable object")

  # Generate IQRoutputTable object
  if (is.null(object)) {
    out__ <- list(
      xtable = xtable,
      xtitle = xtitle,
      xfooter = xfooter,
      report = report
    )
  } else {
    out__ <- list(
      xtable = object$xtable,
      xtitle = {
        if (is.null(xtitle)) {
          x__ <- object$xtitle
        } else {
          x__ <- xtitle
        }
        x__
      },
      xfooter =  {
        if (is.null(xfooter)) {
          x__ <- object$xfooter
        } else {
          x__ <- xfooter
        }
        x__
      },
      report =  {
        if (is.null(report)) {
          x__ <- object$report
        } else {
          x__ <- report
        }
        x__
      }
    )
  }
  class(out__) <- c("IQRoutputTable",class(out__))

  # Write the table in text format to the output file. If the file already exists, it will be overwritten.
  if (!is.null(filename)) {
    aux_filewrite(text=text_IQRoutputTable(out__,report=report,na.string=na.string),filename=paste0(aux_strrep(filename,".txt",""),".txt"))
    # Generate logfile if needed
    genComplianceLog(filename)
  }

  # In any case, return the object
  return(out__)
}


####################
# Overloading "print" for IQRoutputTable objects
####################
#' Overloading "print" for IQRoutputTable objects
#'
#' @param x IQRoutputTable object
#' @param ... Additional parameters
#' @export
print.IQRoutputTable <- function (x, ...) {
  cat(text_IQRoutputTable(x,report = FALSE))
  cat("\n\nIQRoutputTable object")
}


###
# text_IQRoutputTable: convert IQRoutputTable object to text
###
#' Convert IQRoutputTable to text representation
#'
#' @param object IQRoutputTable object
#' @param report TRUE: add reporting tags <TT>, <TR>, <TH>, <TF>
#'               FALSE: do not add (for scfreen output only).
#'               Also stored in object but can be overridden here
#' @param na.string Character string to replace NA entries by. Defaults to NULL.
#' @return Character string with table in text format
#' @export
text_IQRoutputTable <- function (object,report=NULL, na.string=NULL) {
  if (!"IQRoutputTable" %in% class(object)) stop("Input argument not an IQRoutputTable object")

  xtable <- object$xtable
  xfooter <- object$xfooter
  xtitle <- object$xtitle
  if (is.null(report)) report <- object$report

  if (!is.null(xfooter)) {
    if (report) {
      # Replace \n with <br>
      xfooter <- gsub("\n","<br>",xfooter,fixed = TRUE)
      # Replace single * at start or * after \n with space behind by \*
      # to ensure that no itemized list is used in Word
      xfooter <- gsub("^\\* ","\\\\* ",xfooter)
      xfooter <- gsub("<br>\\* ","<br>\\\\* ",xfooter)
    } else {
      # Replace <br> with \n
      xfooter <- gsub("<br>","\n",xfooter,fixed = TRUE)
    }
  }
  ## handle report type
  if (report) {
    trid__ <- "<TR>"
    tfid__ <- "<TF>"
    ttid__ <- "<TT>"
    thid__ <- "<TH>"
  } else {
    trid__ <- ""
    tfid__ <- ""
    ttid__ <- ""
    thid__ <- ""
  }

  ## Convert all numeric entries to characters ... user is responsible to format the
  ## numerics already with signifcant digits
  tr__ <- format(data.frame(lapply(xtable,function (x) as.character(x)),stringsAsFactors = FALSE),justify="left")

  if (!is.null(na.string))
    tr__ <- data.frame(lapply(tr__, function(x) {x[grepl("^NA$", trimws(x))] <- na.string; x} ))

  ## get column names, create them if necessary
  if ("matrix" %in% class(xtable)){
    th__ <- paste0("X",c(1:ncol(xtable)))
  }  else{ #we have a frame
    th__ <- colnames(xtable)
  }

  ##  find column widths: the width for each column is max(1, maxlength(entries), length(colname))
  widths <- rep(1,ncol(xtable))
  for (coli__ in c(1:length(widths))){widths[coli__] <- max(1,max(nchar(tr__[,coli__])),nchar(th__[coli__]))}

  ## prepare the header row
  for (coli__ in c(1:length(th__))){
    th__[coli__] <- format(th__[coli__], width = widths[coli__])
  }
  th__ <- paste(th__,collapse= " | ")
  th__ <- paste(thid__," ",th__, collapse= "")

  ## prepare the table rows.
  # Explanation: widths[colindex]=9 results in sprintf('%-9s,tr__[,colindex]).
  # '%-9s' is '%s' with padding. A negative index adds the spaces to the right of %s (instead of the left)
  for (coli__ in c(1:length(widths))){
    tr__[,coli__] <- sprintf(paste('%-',widths[coli__], 's',sep=''),tr__[,coli__])
  }
  tr__ <- utils::capture.output(utils::write.table(tr__,quote=FALSE,sep=" | ",row.names = FALSE,col.names=FALSE))
  tr__ <- paste(tr__,collapse = paste0("\n",trid__,"   "))
  tr__ <- paste(paste0(trid__,"  "),tr__,collapse="")

  ## prepare the footer and title rows
  if (!is.null(xfooter)) {
    if (report) {
      tf__ <- paste(tfid__,"  ",xfooter, collapse= "")
    } else {
      tf__ <- paste0(tfid__,xfooter, collapse= "")
    }
    tf__ <- format(tf__, width = nchar(th__))
  } else {
    tf__ <- ""
  }

  if (!is.null(xtitle)) {
    tt__ <- paste(ttid__," ",xtitle, collapse="")
    tt__ <- format(tt__, width = nchar(th__))
  } else {
    tt__ <- ""
  }

  ## prepare the separator lines and add them to head and foot
  tablength__ <- max(nchar(th__),nchar(tt__))  # get the maximum length of the table
  nminus <- 7
  if (!report) nminus <- 3

  xxx <- "       "
  if (!report) xxx <- "   "
  thsep__ <- paste0(c(xxx,rep("-",tablength__-nminus),"\n"),collapse="")
  tfsep__ <- paste0(c(xxx,rep("-",tablength__-nminus)),collapse="")
  ttsep__ <- paste0(c(xxx,rep("=",tablength__-nminus),"\n"),collapse="")
  if (is.null(xtitle)) {
    IQRtableHead__ <- paste(th__,thsep__, sep = "\n")
  } else {
    IQRtableHead__ <- paste(tt__,ttsep__,th__,thsep__, sep = "\n")
  }
  if (is.null(xfooter)) {
    IQRtableFoot__ <- NULL
  } else {
    IQRtableFoot__ <- paste(tfsep__,tf__, sep = "\n")
  }

  allTable__ <- paste0(IQRtableHead__,tr__,"\n",IQRtableFoot__)

  allTable__
}

###
# genComplianceLog: generate the compliance log file
#  for given outputfilename
###
genComplianceLog <- function(outputfilename) {

  # Check if compliance mode active
  if (!is_enabled_complianceMode()) return()

  # Check if filename given
  if (is.null(outputfilename)) return()

  # Check if COMPLIANCE_MODE_SCRIPT_NAME exists in globalenv()
  e__ <- globalenv()

  if (!("COMPLIANCE_MODE_SCRIPT_NAME" %in% ls(e__)))
    stop("Compliance mode is enabled but the COMPLIANCE_MODE_SCRIPT_NAME variable has not\nbeen defined in the global environment (by the user). You can use the function IQRinitCompliance to do so")

  # Get the compliance "script" name
  COMPLIANCE_MODE_SCRIPT_NAME <- e__$COMPLIANCE_MODE_SCRIPT_NAME

  ## get information for the logfile
  logt__ <- "<TT>   File generation log"
  logfilepath__ <- paste(outputfilename,'.log',sep='')
  outputfilename <- gsub('//','/',outputfilename)
  userrow__ <- Sys.info()[['user']]
  timerow__ <- Sys.time()

  ## add first column and table delimiters
  callr__   <- paste('<TR>   Analysis file                       | ',COMPLIANCE_MODE_SCRIPT_NAME,sep='')
  pathrow__ <- paste('<TR>   File (relative to calling function) | ',gsub("\\\\","/",outputfilename),sep='')
  userrow__ <- paste('<TR>   Username                            | ',userrow__,sep='')
  timerow__ <- paste('<TR>   Date of creation                    | ',timerow__,sep='')

  ## prepare the separator line for the title
  loglength__ <- max(1,max(nchar(callr__)),nchar(timerow__),nchar(userrow__),nchar(pathrow__))
  logsep__ <- paste(c("       ",rep("=",loglength__-7),"\n"),collapse="",sep='')

  ## write to logfile
  write(c(logt__,logsep__,pathrow__,userrow__,timerow__,callr__),logfilepath__,append=FALSE)
}



