#' Imports NONMEM results and formats it as a parameter estimation table
#'
#' The estimation table can be exported to a table text file, ready for reporting
#' to MS Word with IQReport. Note that this is a standard format for the output table.
#' Other formats can easily be generated programatically or on request.
#' The user can optionally provide annotation information in the NONMEM control file
#' that will be used to annoate the imported results in a reasonable manner.
#' The format of that annotation is described below.
#'
#' NONMEM model annotation is a piece of commented text in a NONMEM control file that
#' can appear at any place in the NONMEM control file but ideally appears at the start.
#' It is bounded by the key words "MODEL ANNOTATION START" and "MODEL ANNOTATION END" which
#' each are only allowed to appear once in the NONMEM control file and the start version should
#' come before the end version.
#'
#' ```
#' ; ==MODEL ANNOTATION START==
#' ; Documentation of the annotation format:
#' ; ---------------------------------------
#' ;  - Only lines starting with "; THETAn", "; ETAn", and "; SIGMAn" are interpreted.
#' ;    The other lines are simply ignored and can be used for further documentation
#' ;    of the model - if desired.
#' ;  - THETA, ETA, and SIGMA information can be provided in any order. Here in this
#' ;    example we order them in blocks.
#' ;  - Definition of THETAs happens as shown below with a comma separated list of
#' ;    information. As the list is comma separated, commata in elements need to be
#' ;    avoided. The entries are documented in the order of required appearance:
#' ;      1) Name: Parameter name
#' ;      2) COV:  1=parameter is a covariate coefficient,
#' ;               0=parameter is not a covariate coefficient
#' ;      3) ERR:  1=parameter is an error model parameter (if coded with THETAS),
#' ;               0=parameter is not an error model parameter
#' ;      4) Unit: Unit of the parameter
#' ;      5) Description: Description of parameter
#' ;      6) Transformation: N=untransformed, L=log transformed, G=logit transformed.
#' ;                 Only used for parameters that are neither covariates nor error model
#' ;                 parameters. Useful if mu-referencing is used as the back transformed
#' ;                 parameters will be determined (including their standard errors and
#' ;                 RSEs). Transformation information is optional and "N" will be assumed
#' ;                 by default.
#' ;
#' ;          Name,        COV, ERR, Unit,     Description,                                 Transformation
#' ; -----------------------------------------------------------------------------------------------------
#' ; THETA1:  Ka,          0,   0,   1/hour,   First-order Absorption Rate Constant,        N
#' ; THETA2:  F1Sty1107,   0,   0,   fraction, Absolute bioavailability for Study 1107,     L
#' ; THETA3:  F1others,    0,   0,   fraction, Absolute bioavailability for All SC Studies, G
#' ; THETA4:  CL,          0,   0,   L/hour,   Central Clearance,                           N
#' ; THETA5:  Vc,          0,   0,   L,        Central Volume of Distribution,              N
#' ; THETA6:  Q,           0,   0,   L/hour,   Distributional Clearance,                    N
#' ; THETA7:  Vp,          0,   0,   L,        Peripheral Volume of Distribution,           N
#' ; THETA8:  betaCLWT,    1,   0,   .,        Power for Weight on CL,
#' ; THETA9:  betaVcWT,    1,   0,   .,        Power for Weight on Vc,
#' ; THETA10: betaF1WT,    1,   0,   .,        Power for Weight on F1,
#' ; THETA11: betaF1AGE,   1,   0,   .,        Power for Age on F1,
#' ; THETA12: betaVpWT,    1,   0,   .,        Power for Weight on Vp,
#' ;
#' ; For ETAs only the parameter name and its description needs to be provided. Again comma
#' ; separated information.
#' ;
#' ;          Name,        Description
#' ; ---------------------------------------------------------------------------
#' ; ETA1:    IIV(Vc),     IIV in Vc [exp]
#' ; ETA2:    IIV(CL),     IIV in CL [exp]
#' ; ETA3:    IIV(Q),      IIV in Q  [exp]
#' ; ETA4:    IIV(Vp),     IIV in Vp [exp]
#' ; ETA5:    IIV(Ka),     IIV in Ka [exp]
#' ;
#' ; The definition of the SIGMA information is not required if THETAs have been used
#' ; to code error model parameters. Otherwise, if error model parameters have been coded
#' ; using SIGMAs, then the annotation information can be provided here.
#' ;
#' ;          Name,        Unit,         Description
#' ; ---------------------------------------------------------------------------
#' ; SIGMA1:  ERR1,        log(ng/mL),   RV (log scale, IV Full-Profile) [add]
#' ; SIGMA2:  ERR2,        log(ng/mL),   RV (log scale, IV Phase 2/3) [add]
#' ; SIGMA3:  ERR3,        log(ng/mL),   RV (log scale, SC Full-Profile) [add]
#' ; SIGMA4:  ERR4,        log(ng/mL),   RV (log scale, SC Phase 3) [add]
#' ; ==MODEL ANNOTATION END==
#' ```
#' @param NMcontrolFile Path to the NONMEM control file
#' @param NMxmlFile Path to the NONMEM XML output file. If not provided it will
#'   be assumes that the XML file is in the same folder as the control file and
#'   that the difference in name is just the extension (.xml)
#' @param FLAGtransform MU referencing requires the use of transformed THETAs for
#'   fixed effects of typical values. If this flag is set to TRUE, back transformation
#'   is done to report the parameters in the domain of human interest. If FALSE is used
#'   then the parameters are reported as estimated by NONMEM. Correct back transformation
#'   requires providing the "Transformation" information in the model annotation.
#'   "N" for untransformed (default), "L" for log transformed, and "G" for logit transformed.
#' @param SIGNIF Number of significant digits for the values, RSEs, and shrinkage. The
#'   objective function, AIC, BIC will be rounded to the closest integer.
#' @param filename File name with path to where to store the exported table file.
#' @return List with parsed information - self explaining
#' @export
#' @md
#' @examples
#' setwd("~")
#'
#' # An example NONMEM model is provided in the package - in the same folder is also
#' # the model.xml NONMEM output in XML format that is used to load the results
#' NMcontrolFile <- system.file(package = "IQReportAPI","example/NONMEM/model.nmctl")
#'
#' # Have a look at the annotation and the model
#' file.edit(NMcontrolFile)
#'
#' # Generate a parameters table
#' table <- tableNM_IQReportAPI(NMcontrolFile)
#'
#' # Look at the table
#' table
#'
#' # Export table to file
#' tableNM_IQReportAPI(NMcontrolFile,filename="TAB01_NONMEM_Result.txt")
#'
#' # Generate a simple Word file with this table
#' rmd <- rmdTABLE(file = "TAB01_NONMEM_Result.txt",fontsize = 9) + rmdNOINTRO()
#' export_IQRrmd(rmd,"exampleNM.rmd")
#' IQReport("exampleNM.rmd")
#'
#' # Done ... that was easy - Enjoy!
tableNM_IQReportAPI <- function (NMcontrolFile,NMxmlFile=NULL,FLAGtransform=TRUE,SIGNIF=3,filename=NULL) {

  NMresults <- importNMresults_IQReportAPI(NMcontrolFile,NMxmlFile,FLAGtransform)

  OFV <- NMresults$objectiveFunction$OBJ
  AIC <- NMresults$objectiveFunction$AIC
  BIC <- NMresults$objectiveFunction$BIC

  TI  <- NMresults$terminationInfo

  FE  <- NMresults$parameters$FE_TV
  RE  <- NMresults$parameters$RE_SD
  COR <- NMresults$parameters$RE_CORR
  COV <- NMresults$parameters$COV
  ERR <- NMresults$parameters$ERR_SD

  EMTPYrow <- data.frame(
    PARAMETER = " ",
    VALUE     = " ",
    RSE       = " ",
    SHRINKAGE = " ",
    COMMENT   = " ",
    ESTIMATED = 1,
    stringsAsFactors = FALSE
  )

  FEout <- rbind(
    data.frame(
      PARAMETER = c("**Typical parameters**",FE$names),
      VALUE     = c(" ",signif(FE$values,SIGNIF)),
      RSE       = c(" ",signif(FE$rse,SIGNIF)),
      SHRINKAGE = c(" ",rep("-",length(FE$names))),
      COMMENT   = c(" ",paste0(FE$description," (",FE$unit,")")),
      ESTIMATED = c(1,FE$estimated),
      stringsAsFactors = FALSE
    ),
    EMTPYrow
  )


  REout <- rbind(
    data.frame(
      PARAMETER = c("**Inter-individual variability**",RE$names),
      VALUE     = c(" ",signif(RE$values,SIGNIF)),
      RSE       = c(" ",signif(RE$rse,SIGNIF)),
      SHRINKAGE = c(" ",paste0(signif(RE$shringkageSDpercent,SIGNIF),"%")),
      COMMENT   = c(" ",paste0(RE$description)),
      ESTIMATED = c(1,RE$estimated),
      stringsAsFactors = FALSE
    ),
    EMTPYrow
  )

  if (length(COR$names) > 1) {
    CORout <- rbind(
      data.frame(
        PARAMETER = c("**Correlation of random effects**",COR$names),
        VALUE     = c(" ",signif(COR$values,SIGNIF)),
        RSE       = c(" ",signif(COR$rse,SIGNIF)),
        SHRINKAGE = c(" ",rep("-",length(COR$names))),
        COMMENT   = c(" ",rep("Correlation coefficient",length(COR$names))),
        ESTIMATED = c(1,COR$estimated),
        stringsAsFactors = FALSE
      ),
      EMTPYrow
    )
  } else {
    CORout <- NULL
  }

  if (length(COV$names) > 1) {
    COVout <- rbind(
      data.frame(
        PARAMETER = c("**Parameter-Covariate relationships**",COV$names),
        VALUE     = c(" ",signif(COV$values,SIGNIF)),
        RSE       = c(" ",signif(COV$rse,SIGNIF)),
        SHRINKAGE = c(" ",rep("-",length(COV$names))),
        COMMENT   = c(" ",paste0(COV$description," (",COV$unit,")")),
        ESTIMATED = c(1,COV$estimated),
        stringsAsFactors = FALSE
      ),
      EMTPYrow
    )
  } else {
    COVout <- NULL
  }

  if (length(ERR$names) > 1) {
    ERRout <- rbind(
      data.frame(
        PARAMETER = c("**Residual Variability**",ERR$names),
        VALUE     = c(" ",signif(ERR$values,SIGNIF)),
        RSE       = c(" ",signif(ERR$rse,SIGNIF)),
        SHRINKAGE = c(" ",rep("-",length(ERR$names))),
        COMMENT   = c(" ",paste0(ERR$description," (",ERR$unit,")")),
        ESTIMATED = c(1,ERR$estimated),
        stringsAsFactors = FALSE
      ),
      EMTPYrow
    )
  } else {
    ERRout <- NULL
  }

  OFVout <- data.frame(
    PARAMETER = c("Objective function","AIC","BIC"),
    VALUE     = c(round(OFV),round(AIC),round(BIC)),
    RSE       = c("-","-","-"),
    SHRINKAGE = c("-","-","-"),
    COMMENT   = c("-","-","-"),
    ESTIMATED = c(1, 1, 1),
    stringsAsFactors = FALSE
  )

  ALLout <- rbind(
    FEout,
    REout,
    CORout,
    COVout,
    ERRout,
    OFVout
  )

  # Handle estimated information
  ix <- which(ALLout$ESTIMATED==0)
  ALLout$RSE[ix] <- "-"
  ALLout$SHRINKAGE[ix] <- "-"
  ALLout$VALUE[ix] <- paste0(ALLout$VALUE[ix]," (FIX)")

  # Remove estimated
  ALLout$ESTIMATED <- NULL

  # Generate table
  title <- "Population parameter estimates"
  footer <- paste0(
    "Model: ",gsub(pattern = "\\\\",replacement = "/",x = NMcontrolFile),"\n",
    "Significant digits: ",SIGNIF," (Objective function rounded to closest integer value)\n",
    "Inter-individual variability and residual variability reported as standard deviation"
  )
  table <- IQRoutputTable(xtable = ALLout,
                          xtitle = title,
                          xfooter = footer,
                          filename = filename)

  if (!is.null(filename)) {
    IQRoutputTable(object = table,report=TRUE,filename=filename)
  }

  return(table)
}

#' Imports NONMEM results from NONMEM XML file
#'
#' The user can optionally provide annotation information in the NONMEM control file
#' that will be used to annoate the imported results in a reasonable manner.
#' The format of that annotation is described below.
#'
#' NONMEM model annotation is a piece of commented text in a NONMEM control file that
#' can appear at any place in the NONMEM control file but ideally appears at the start.
#' It is bounded by the key words "MODEL ANNOTATION START" and "MODEL ANNOTATION END" which
#' each are only allowed to appear once in the NONMEM control file and the start version should
#' come before the end version.
#'
#' ```
#' ; ==MODEL ANNOTATION START==
#' ; Documentation of the annotation format:
#' ; ---------------------------------------
#' ;  - Only lines starting with "; THETAn", "; ETAn", and "; SIGMAn" are interpreted.
#' ;    The other lines are simply ignored and can be used for further documentation
#' ;    of the model - if desired.
#' ;  - THETA, ETA, and SIGMA information can be provided in any order. Here in this
#' ;    example we order them in blocks.
#' ;  - Definition of THETAs happens as shown below with a comma separated list of
#' ;    information. As the list is comma separated, commata in elements need to be
#' ;    avoided. The entries are documented in the order of required appearance:
#' ;      1) Name: Parameter name
#' ;      2) COV:  1=parameter is a covariate coefficient,
#' ;               0=parameter is not a covariate coefficient
#' ;      3) ERR:  1=parameter is an error model parameter (if coded with THETAS),
#' ;               0=parameter is not an error model parameter
#' ;      4) Unit: Unit of the parameter
#' ;      5) Description: Description of parameter
#' ;      6) Transformation: N=untransformed, L=log transformed, G=logit transformed.
#' ;                 Only used for parameters that are neither covariates nor error model
#' ;                 parameters. Useful if mu-referencing is used as the back transformed
#' ;                 parameters will be determined (including their standard errors and
#' ;                 RSEs). Transformation information is optional and "N" will be assumed
#' ;                 by default.
#' ;
#' ;          Name,        COV, ERR, Unit,     Description,                                 Transformation
#' ; -----------------------------------------------------------------------------------------------------
#' ; THETA1:  Ka,          0,   0,   1/hour,   First-order Absorption Rate Constant,        N
#' ; THETA2:  F1Sty1107,   0,   0,   fraction, Absolute bioavailability for Study 1107,     L
#' ; THETA3:  F1others,    0,   0,   fraction, Absolute bioavailability for All SC Studies, G
#' ; THETA4:  CL,          0,   0,   L/hour,   Central Clearance,                           N
#' ; THETA5:  Vc,          0,   0,   L,        Central Volume of Distribution,              N
#' ; THETA6:  Q,           0,   0,   L/hour,   Distributional Clearance,                    N
#' ; THETA7:  Vp,          0,   0,   L,        Peripheral Volume of Distribution,           N
#' ; THETA8:  betaCLWT,    1,   0,   .,        Power for Weight on CL,
#' ; THETA9:  betaVcWT,    1,   0,   .,        Power for Weight on Vc,
#' ; THETA10: betaF1WT,    1,   0,   .,        Power for Weight on F1,
#' ; THETA11: betaF1AGE,   1,   0,   .,        Power for Age on F1,
#' ; THETA12: betaVpWT,    1,   0,   .,        Power for Weight on Vp,
#' ;
#' ; For ETAs only the parameter name and its description needs to be provided. Again comma
#' ; separated information.
#' ;
#' ;          Name,        Description
#' ; ---------------------------------------------------------------------------
#' ; ETA1:    IIV(Vc),     IIV in Vc [exp]
#' ; ETA2:    IIV(CL),     IIV in CL [exp]
#' ; ETA3:    IIV(Q),      IIV in Q  [exp]
#' ; ETA4:    IIV(Vp),     IIV in Vp [exp]
#' ; ETA5:    IIV(Ka),     IIV in Ka [exp]
#' ;
#' ; The definition of the SIGMA information is not required if THETAs have been used
#' ; to code error model parameters. Otherwise, if error model parameters have been coded
#' ; using SIGMAs, then the annotation information can be provided here.
#' ;
#' ;          Name,        Unit,         Description
#' ; ---------------------------------------------------------------------------
#' ; SIGMA1:  ERR1,        log(ng/mL),   RV (log scale, IV Full-Profile) [add]
#' ; SIGMA2:  ERR2,        log(ng/mL),   RV (log scale, IV Phase 2/3) [add]
#' ; SIGMA3:  ERR3,        log(ng/mL),   RV (log scale, SC Full-Profile) [add]
#' ; SIGMA4:  ERR4,        log(ng/mL),   RV (log scale, SC Phase 3) [add]
#' ; ==MODEL ANNOTATION END==
#' ```
#' @param NMcontrolFile Path to the NONMEM control file
#' @param NMxmlFile Path to the NONMEM XML output file. If not provided it will
#'   be assumes that the XML file is in the same folder as the control file and
#'   that the difference in name is just the extension (.xml)
#' @param FLAGtransform MU referencing requires the use of transformed THETAs for
#'   fixed effects of typical values. If this flag is set to TRUE, back transformation
#'   is done to report the parameters in the domain of human interest. If FALSE is used
#'   then the parameters are reported as estimated by NONMEM. Correct back transformation
#'   requires providing the "Transformation" information in the model annotation.
#'   "N" for untransformed (default), "L" for log transformed, and "G" for logit transformed.
#' @return List with parsed information - self explaining
#' @export
#' @md
importNMresults_IQReportAPI <- function(NMcontrolFile,NMxmlFile=NULL,FLAGtransform=TRUE) {

  # Try to get XML file assuming its in the same folder as the control file
  if (is.null(NMxmlFile)) {
    x <- aux_fileparts(NMcontrolFile)
    NMxmlFile <- file.path(x$pathname,paste0(x$filename,".xml"))
  }

  # Check presence of files
  if (!file.exists(NMcontrolFile)) {
    stop("NONMEM control file could not be found")
  }
  if (!file.exists(NMxmlFile)) {
    stop("NONMEM XML output file could not be found")
  }

  # Parse first the XML file
  RESULTS__ <- parseNONMEMxmlFile(NMxmlFile)
  RESULTS__$modelFile <- NMcontrolFile

  # Get number THETAS, ETAS, SIGMAS
  NTHETA__ <- length(RESULTS__$THETA$names)
  NETA__ <- nrow(RESULTS__$OMEGA2$matrix)
  NSIGMA__ <- nrow(RESULTS__$SIGMA2$matrix)

  # Now parse the project header and return info (user provided or default)
  PROJECTINFO <- parseNONMEMprojectHeader(NMcontrolFile,NTHETA__,NETA__,NSIGMA__)

  # Initialize output structure
  output__                          <- list()
  output__$termination_info         <- RESULTS__$termination_info
  output__$modelFile                <- RESULTS__$modelFile
  output__$xmlFile                  <- RESULTS__$xmlFile
  output__$objectivefunction        <- c()
  output__$rawParameterInfo         <- list()
  output__$PROJECTINFO              <- PROJECTINFO

  OFVvalue                        <- RESULTS__$objectivefunction
  NRPARAMETERS_ESTIMATED__        <- as.double(RESULTS__$NRPARAM_ESTIMATED)
  NROBSERVATIONS                  <- as.double(RESULTS__$NROBSERVATIONS)
  AIC                             <- calcAICBIC(OFV = OFVvalue,
                                                nData = NROBSERVATIONS,
                                                nParam = NRPARAMETERS_ESTIMATED__)$AIC
  BIC                             <- calcAICBIC(OFV = OFVvalue,
                                                nData = NROBSERVATIONS,
                                                nParam = NRPARAMETERS_ESTIMATED__)$BIC
  output__$objectivefunction$OBJ    <- OFVvalue
  output__$objectivefunction$AIC    <- AIC
  output__$objectivefunction$BIC    <- BIC
  if (is.na(OFVvalue)) return(output__)

  names_real                      <- c()
  names                           <- RESULTS__$THETA$names
  theta_ix__                      <- 1:length(names)
  names_real__                    <- PROJECTINFO$THETANAMES[theta_ix__]
  RESULTS__$THETA$names_real__    <- names_real__


  # Change names for OMEGA2
  names_real__                      <- c()
  names_real_alternative__          <- c()
  names                           <- RESULTS__$OMEGA2$names;
  for (k__ in seq_along(names)) {
    n                           <- names[k__]
    n                           <- aux_strrep(n,'OMEGA2','')
    n                           <- aux_strrep(n,'_','')
    terms                       <- aux_explodePC(n)
    row                         <- as.double(terms[1])
    col                         <- as.double(terms[2])
    if (row==col) {
      names_real__                <- c(names_real__,sprintf('omega2(%s)',PROJECTINFO$ETANAMES[row]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('omega2(%s)',PROJECTINFO$ETANAMES[row]))
    } else {
      names_real__                <- c(names_real__,sprintf('omega2(%s,%s)',PROJECTINFO$ETANAMES[row],PROJECTINFO$ETANAMES[col]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('omega2(%s,%s)',PROJECTINFO$ETANAMES[col],PROJECTINFO$ETANAMES[row]))
    }
  }
  RESULTS__$OMEGA2$names_real__               <- names_real__
  RESULTS__$OMEGA2$names_real_alternative__   <- names_real_alternative__


  # Change names for OMEGAC
  names_real__                      <- c()
  names_real_alternative__          <- c()
  names                           <- RESULTS__$OMEGAC$names;
  for (k__ in seq_along(names)) {
    n                           <- names[k__]
    n                           <- aux_strrep(n,'OMEGAC','')
    n                           <- aux_strrep(n,'_','')
    terms                       <- aux_explodePC(n)
    row                         <- as.double(terms[1])
    col                         <- as.double(terms[2])
    if (row==col) {
      names_real__                <- c(names_real__,sprintf('omega(%s)',PROJECTINFO$ETANAMES[row]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('omega(%s)',PROJECTINFO$ETANAMES[row]))
    } else {
      names_real__                <- c(names_real__,sprintf('corr(%s,%s)',PROJECTINFO$ETANAMES[row],PROJECTINFO$ETANAMES[col]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('corr(%s,%s)',PROJECTINFO$ETANAMES[col],PROJECTINFO$ETANAMES[row]))
    }
  }
  RESULTS__$OMEGAC$names_real__               <- names_real__
  RESULTS__$OMEGAC$names_real_alternative__   <- names_real_alternative__

  # Change names for SIGMA2
  names_real__                      <- c()
  names_real_alternative__          <- c()
  names                           <- RESULTS__$SIGMA2$names
  for (k__ in seq_along(names)) {
    n                           <- names[k__]
    n                           <- aux_strrep(n,'SIGMA2','')
    n                           <- aux_strrep(n,'_','')
    terms                       <- aux_explodePC(n)
    row                         <- as.double(terms[1])
    col                         <- as.double(terms[2])
    if (row==col) {
      names_real__                <- c(names_real__,sprintf('sigma2(%s)',PROJECTINFO$SIGMANAMES[row]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('sigma2(%s)',PROJECTINFO$SIGMANAMES[row]))
    } else {
      names_real__                <- c(names_real__,sprintf('sigma2(%s,%s)',PROJECTINFO$SIGMANAMES[row],PROJECTINFO$SIGMANAMES[col]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('sigma2(%s,%s)',PROJECTINFO$SIGMANAMES[col],PROJECTINFO$SIGMANAMES[row]))
    }
  }
  RESULTS__$SIGMA2$names_real__               <- names_real__
  RESULTS__$SIGMA2$names_real_alternative__   <- names_real_alternative__

  # Change names for SIGMAC
  names_real__                      <- c()
  names_real_alternative__          <- c()
  names                           <- RESULTS__$SIGMAC$names;
  for (k__ in seq_along(names)) {
    n                           <- names[k__]
    n                           <- aux_strrep(n,'SIGMAC','')
    n                           <- aux_strrep(n,'_','')
    terms                       <- aux_explodePC(n)
    row                         <- as.double(terms[1])
    col                         <- as.double(terms[2])
    if (row==col) {
      names_real__                <- c(names_real__,sprintf('sigma(%s)',PROJECTINFO$SIGMANAMES[row]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('sigma(%s)',PROJECTINFO$SIGMANAMES[row]))
    } else {
      names_real__                <- c(names_real__,sprintf('corr(%s,%s)',PROJECTINFO$SIGMANAMES[row],PROJECTINFO$SIGMANAMES[col]))
      names_real_alternative__    <- c(names_real_alternative__, sprintf('corr(%s,%s)',PROJECTINFO$SIGMANAMES[col],PROJECTINFO$SIGMANAMES[row]))
    }
  }
  RESULTS__$SIGMAC$names_real__               <- names_real__
  RESULTS__$SIGMAC$names_real_alternative__   <- names_real_alternative__

  # rawParameters field: information only for displaying but not for sampling

  # Create the rawParameterInfo field - fixedEffects
  # Determine fixed effect names etc (THETANAMES for THETACOVS==0)
  FENAMES <- PROJECTINFO$THETANAMES[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0]
  FEESTIMATED <- RESULTS__$THETA$estimated[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0]
  FEVALUES <- unname(RESULTS__$THETA$values[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0])
  FESTDERR <- unname(RESULTS__$THETA$standarderror[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0])
  FESTDERR[FEESTIMATED==0] <- NA
  FERSE <- unname( 100*FESTDERR/FEVALUES)
  FERSE[FERSE==0] <- NA
  fixedEffects__                <- list()
  fixedEffects__$names          <- FENAMES
  fixedEffects__$description    <- PROJECTINFO$THETADESCRIPTION[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0]
  fixedEffects__$unit           <- PROJECTINFO$THETAUNIT[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0]
  fixedEffects__$transformation <- PROJECTINFO$THETATRANS[PROJECTINFO$THETACOVS==0 & PROJECTINFO$THETAERRS==0]
  fixedEffects__$estimated      <- FEESTIMATED
  fixedEffects__$values         <- FEVALUES
  fixedEffects__$stderr         <- FESTDERR
  fixedEffects__$rse            <- FERSE
  output__$rawParameterInfo$fixedEffects                    <- fixedEffects__

  # Create the rawParameterInfo field - randomEffects
  # Based on RESULTS__$OMEGAC
  randomEffects <- list()
  randomEffects$names           <- PROJECTINFO$ETANAMES
  randomEffects$description     <- PROJECTINFO$ETADESCRIPTION
  randomEffects$values          <- c()
  randomEffects$estimated       <- c()
  randomEffects$stderr          <- c()
  randomEffects$rse             <- c()
  for (k__ in seq_along(randomEffects$names)) {
    # Determine if parameter was estimated
    ix__ <- which(randomEffects$names[k__]==PROJECTINFO$ETANAMES)
    randomEffects$estimated[k__] <- RESULTS__$OMEGAC$estimatedMATRIX[ix__,ix__]

    ix__ <- which(paste0("omega(",randomEffects$names[k__],")")==RESULTS__$OMEGAC$names_real__)

    randomEffects$values[k__]  <- RESULTS__$OMEGAC$values[ix__]

    if (randomEffects$estimated[k__]==1) {
      randomEffects$stderr[k__]  <- RESULTS__$OMEGAC$standarderror[ix__]
      randomEffects$rse[k__]     <- abs(100*randomEffects$stderr[k__]/randomEffects$values[k__])
    } else {
      randomEffects$stderr[k__]  <- NA
      randomEffects$rse[k__]     <- NA
    }
  }
  randomEffects$shringkageSDpercent <- RESULTS__$ETA_SHRINKAGE
  # Add to output__
  output__$rawParameterInfo$randomEffects <- randomEffects

  # Create the rawParameterInfo field - correlation
  ixCorr__ <- which(grepl("corr(",RESULTS__$OMEGAC$names_real__,fixed = TRUE) & RESULTS__$OMEGAC$estimated)

  correlation <- list()
  correlation$names <- RESULTS__$OMEGAC$names_real_alternative__[ixCorr__]
  correlation$values <- unname(RESULTS__$OMEGAC$values[ixCorr__])
  correlation$estimated <- unname(RESULTS__$OMEGAC$estimated[ixCorr__])
  correlation$stderr <- unname(RESULTS__$OMEGAC$standarderror[ixCorr__])
  correlation$rse <- unname(abs(100*correlation$stderr/correlation$values))

  # Add to output__
  output__$rawParameterInfo$correlation = correlation

  # Create the rawParameterInfo field - covariate
  covariate <- list()
  covariate$names               <- unname(RESULTS__$THETA$names_real__[PROJECTINFO$THETACOVS==1])
  covariate$description    <- PROJECTINFO$THETADESCRIPTION[PROJECTINFO$THETACOVS==1]
  covariate$unit           <- PROJECTINFO$THETAUNIT[PROJECTINFO$THETACOVS==1]
  covariate$transformation <- PROJECTINFO$THETATRANS[PROJECTINFO$THETACOVS==1]
  covariate$values <- unname(RESULTS__$THETA$values[PROJECTINFO$THETACOVS==1])
  covariate$stderr <- unname(RESULTS__$THETA$standarderror[PROJECTINFO$THETACOVS==1])
  covariate$estimated <- unname(RESULTS__$THETA$estimated[PROJECTINFO$THETACOVS==1])
  covariate$rse <- unname(abs(100*covariate$stderr/covariate$values))
  covariate$stderr[covariate$estimated==0] <- NA
  covariate$rse[covariate$estimated==0] <- NA
  # Add to output__
  output__$rawParameterInfo$covariate <- covariate

  # Handle error model parameters ... two cases:
  # via THETA and via SIGMA

  # via THETA
  errorParameterTHETA <- list()
  errorParameterTHETA$names <- unname(RESULTS__$THETA$names_real__[PROJECTINFO$THETAERRS==1])
  errorParameterTHETA$description    <- PROJECTINFO$THETADESCRIPTION[PROJECTINFO$THETAERRS==1]
  errorParameterTHETA$unit           <- PROJECTINFO$THETAUNIT[PROJECTINFO$THETAERRS==1]
  errorParameterTHETA$transformation <- PROJECTINFO$THETATRANS[PROJECTINFO$THETAERRS==1]
  errorParameterTHETA$values <- unname(RESULTS__$THETA$values[PROJECTINFO$THETAERRS==1])
  errorParameterTHETA$stderr <- unname(RESULTS__$THETA$standarderror[PROJECTINFO$THETAERRS==1])
  errorParameterTHETA$estimated <- unname(RESULTS__$THETA$estimated[PROJECTINFO$THETAERRS==1])
  errorParameterTHETA$rse <- unname(abs(100*errorParameterTHETA$stderr/errorParameterTHETA$values))
  errorParameterTHETA$stderr[errorParameterTHETA$estimated==0] <- NA
  errorParameterTHETA$rse[errorParameterTHETA$estimated==0] <- NA

  # via SIGMA
  # Based on RESULTS__$SIGMAC
  errorParameterSIGMA <- list()
  errorParameterSIGMA$names = PROJECTINFO$SIGMANAMES
  errorParameterSIGMA$description    <- PROJECTINFO$SIGMADESCRIPTION
  errorParameterSIGMA$unit           <- PROJECTINFO$SIGMAUNIT
  errorParameterSIGMA$transformation <- rep("N",NSIGMA__)
  errorParameterSIGMA$values <- c()
  errorParameterSIGMA$estimated <- c()
  errorParameterSIGMA$stderr <- c()
  errorParameterSIGMA$rse <- c()
  for (k__ in seq_along(errorParameterSIGMA$names)) {
    # Determine if parameter was estimated
    ix__ <- which(errorParameterSIGMA$names[k__]==PROJECTINFO$SIGMANAMES)
    errorParameterSIGMA$estimated[k__] <- RESULTS__$SIGMAC$estimatedMATRIX[ix__,ix__]

    ix__ <- which(paste0("sigma(",errorParameterSIGMA$names[k__],")")==RESULTS__$SIGMAC$names_real__)

    errorParameterSIGMA$values[k__]  <- RESULTS__$SIGMAC$values[ix__]

    if (errorParameterSIGMA$estimated[k__]==1) {
      errorParameterSIGMA$stderr[k__]  <- RESULTS__$SIGMAC$standarderror[ix__]
      errorParameterSIGMA$rse[k__]     <- abs(100*errorParameterSIGMA$stderr[k__]/errorParameterSIGMA$values[k__])
    } else {
      errorParameterSIGMA$stderr[k__]  <- NA
      errorParameterSIGMA$rse[k__]     <- NA
    }
  }

  # combine
  FLAGincludeSIGMA <- TRUE
  if (all(errorParameterSIGMA$values==1)) {
    if (all(errorParameterSIGMA$estimated==0)) {
      FLAGincludeSIGMA <- FALSE
    }
  }
  errorParameter <- list()
  if (FLAGincludeSIGMA) {
    errorParameter$names <- c(errorParameterTHETA$names,errorParameterSIGMA$names)
    errorParameter$description    <- c(errorParameterTHETA$description,errorParameterSIGMA$description)
    errorParameter$unit           <- c(errorParameterTHETA$unit,errorParameterSIGMA$unit)
    errorParameter$transformation <- c(errorParameterTHETA$transformation,errorParameterSIGMA$transformation)
    errorParameter$values <- c(errorParameterTHETA$values,errorParameterSIGMA$values)
    errorParameter$stderr <- c(errorParameterTHETA$stderr,errorParameterSIGMA$stderr)
    errorParameter$estimated <- c(errorParameterTHETA$estimated,errorParameterSIGMA$estimated)
    errorParameter$rse <- c(errorParameterTHETA$rse,errorParameterSIGMA$rse)
  } else {
    errorParameter <- errorParameterTHETA
  }
  # Add to output__
  output__$rawParameterInfo$errorParameter <- errorParameter

  # Generate new output structure
  out__ <- list(
    modelFile = output__$modelFile,
    xmlFile = output__$xmlFile,
    terminationInfo = output__$termination_info,
    objectiveFunction = output__$objectivefunction,
    FLAGtransform = FLAGtransform,
    parameters = list(
      FE_TV     = data.frame(output__$rawParameterInfo$fixedEffects, stringsAsFactors = FALSE),
      RE_SD   = data.frame(output__$rawParameterInfo$randomEffects, stringsAsFactors = FALSE),
      RE_CORR = data.frame(output__$rawParameterInfo$correlation, stringsAsFactors = FALSE),
      COV     = data.frame(output__$rawParameterInfo$covariate, stringsAsFactors = FALSE),
      ERR_SD  = data.frame(output__$rawParameterInfo$errorParameter, stringsAsFactors = FALSE)
    )
  )

  # Last thing: Transform if needed - and check if transformation allowed
  # Allowed different from N only for FE_TV
  if (!all(out__$parameters$COV$transformation == "N")) {
    stop("Transformation different from 'N' for some covariate parameters. This is not allowed")
  }
  if (!all(out__$parameters$ERR_SD$transformation == "N")) {
    stop("Transformation different from 'N' for some error model parameters. This is not allowed")
  }
  if (!all(out__$parameters$FE_TV$transformation %in% c("N","L","G"))) {
    stop("Transformations of model parameters need to be either 'N', 'L', or 'G'")
  }
  # Handle tranformation FE_TV
  if (FLAGtransform) {
    FETV__ <- out__$parameters$FE_TV
    for (k__ in 1:nrow(FETV__)) {
      trans__ <- FETV__$transformation[k__]
      if (trans__=="N") transf__ <- "phi__"
      if (trans__=="L") transf__ <- "exp(phi__)"
      if (trans__=="G") transf__ <- "exp(phi__)/(1+exp(phi__))"

      phi__ <- FETV__$values[k__]
      FETV__$values[k__] <- eval(parse(text=transf__))
      stderr__ <- FETV__$stderr[k__]
      if (!is.na(stderr__)) {
        jackk__ <- stats::D(parse(text = transf__), "phi__")
        FETV__$stderr[k__] <- abs(eval(jackk__))*stderr__
        FETV__$rse[k__] <- 100*FETV__$stderr[k__]/FETV__$values[k__]
      }
    }
    out__$parameters$FE_TV <- FETV__
  }

  # Return result
  return(out__)
}


# Taking the result of the last $EST statement
parseNONMEMxmlFile <- function(NMxmlFile) {
  if (!file.exists(NMxmlFile)) {
    stop("NONMEM XML output file could not be found")
  }
  output__                          <- list()
  output__$modelFile                <- NA
  output__$xmlFile                  <- NMxmlFile
  content__  <- aux_fileread(NMxmlFile)
  content__  <- aux_strrep(content__,'nm:','')
  ixstart__  <- aux_strFindAll(content__,'<termination_information>')$end
  ixend__    <- aux_strFindAll(content__,'</termination_information>')$start
  if (is.null(ixstart__)) stop("No termination information in XML file")
  termination_info <- c()
  for (k__ in seq_along(1:min(length(ixstart__),length(ixend__)))) {
    x <- substr(content__,ixstart__[k__]+1,ixend__[k__]-1)
    x <- aux_strrep(x,'<![CDATA[','')
    x <- aux_strtrim(aux_strrep(x,']]>',''))
    x <- aux_explode(x,"\n")
    y <- ''
    for (k2__ in seq_along(x)) y <- sprintf('%s    %s\n',y,aux_strtrim(x[k2__]))
    termination_info[k__] <- y
  }
  output__$termination_info <- termination_info;
  ixstart__  <- aux_strFindAll(content__,'<final_objective_function>')$end
  ixend__    <- aux_strFindAll(content__,'</final_objective_function>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    OFVvalue <- substr(content__,ixstart__+1,ixend__-1)
  } else {
    OFVvalue <- NA
  }
  output__$objectivefunction <- as.double(OFVvalue)
  m <- gregexpr("data_nrec='[0-9]+'",content__)
  c <- regmatches(content__,m)
  c <- gsub("data_nrec='","",c)
  c <- as.numeric(gsub("'","",c))
  output__$NROBSERVATIONS <- c
  # THETA
  ixstart__    <- aux_strFindAll(content__,'<theta>')$end
  ixend__      <- aux_strFindAll(content__,'</theta>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    theta_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    theta_text__ <- aux_strrep(theta_text__,'</val>','')
    theta_text__ <- aux_strrep(theta_text__,sprintf("<val name='"),'')
    theta_text__ <- gsub(x=theta_text__,pattern="[0-9]+'>","")
    theta_text__ <- paste0("c(",aux_strrep(theta_text__,"\n",","),")")
    theta_values <- eval(parse(text=theta_text__))
    theta_names <- paste0("THETA",1:length(theta_values))
  } else {
    theta_values <- NULL
    theta_names <- NULL
  }
  output__$THETA$names <- theta_names
  output__$THETA$values <- as.double(theta_values)
  names(output__$THETA$names) <- theta_names
  names(output__$THETA$values) <- theta_names
  # OMEGA
  ixstart__    <- aux_strFindAll(content__,'<omega>')$end
  ixend__      <- aux_strFindAll(content__,'</omega>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    omega_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    omega_text__ <- aux_strrep(omega_text__,'</row>','#')
    omega_text__ <- aux_strrep(omega_text__,'</col>','')
    omega_text__ <- aux_strrep(omega_text__,'<row rname=','')
    omega_text__ <- aux_strrep(omega_text__,'<col cname=','')
    omega_text__ <- aux_strrep(omega_text__,"'",'')
    omega_text__ <- gsub(x=omega_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(omega_text__,'#'), function(x) aux_strtrim(x)))
    OMEGA2 <- matrix(NA,nrow=length(terms),ncol=length(terms))
    OMEGA2_names__  <- c()
    OMEGA2_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        OMEGA2[row,col] <- colterms__[col]
        OMEGA2[col,row] <- colterms__[col]
        OMEGA2_names__    <- c(OMEGA2_names__, sprintf('OMEGA2_%d,%d_',row,col))
        OMEGA2_values__   <- c(OMEGA2_values__, colterms__[col])
      }
    }
  } else {
    OMEGA2 <- NULL
    OMEGA2_names__ <- NULL
    OMEGA2_values__ <- NULL
  }
  output__$OMEGA2$names   <- OMEGA2_names__
  output__$OMEGA2$values  <- as.double(OMEGA2_values__)
  names(output__$OMEGA2$names)   <- OMEGA2_names__
  names(output__$OMEGA2$values)  <- OMEGA2_names__
  output__$OMEGA2$matrix  <- matrix(as.double(OMEGA2),ncol=length(terms),nrow=length(terms))
  # SIGMA
  ixstart__    <- aux_strFindAll(content__,'<sigma>')$end
  ixend__      <- aux_strFindAll(content__,'</sigma>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    sigma_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    sigma_text__ <- aux_strrep(sigma_text__,'</row>','#')
    sigma_text__ <- aux_strrep(sigma_text__,'</col>','')
    sigma_text__ <- aux_strrep(sigma_text__,'<row rname=','')
    sigma_text__ <- aux_strrep(sigma_text__,'<col cname=','')
    sigma_text__ <- aux_strrep(sigma_text__,"'",'')
    sigma_text__ <- gsub(x=sigma_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(sigma_text__,'#'), function(x) aux_strtrim(x)))
    SIGMA2 <- matrix(NA,nrow=length(terms),ncol=length(terms))
    SIGMA2_names__  <- c()
    SIGMA2_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        SIGMA2[row,col] <- colterms__[col]
        SIGMA2[col,row] <- colterms__[col]
        SIGMA2_names__    <- c(SIGMA2_names__, sprintf('SIGMA2_%d,%d_',row,col))
        SIGMA2_values__   <- c(SIGMA2_values__, colterms__[col])
      }
    }
  } else {
    SIGMA2 <- NULL
    SIGMA2_names__ <- NULL
    SIGMA2_values__ <- NULL
  }
  output__$SIGMA2$names   <- SIGMA2_names__
  output__$SIGMA2$values  <- as.double(SIGMA2_values__)
  names(output__$SIGMA2$names)   <- SIGMA2_names__
  names(output__$SIGMA2$values)  <- SIGMA2_names__
  output__$SIGMA2$matrix  <- matrix(as.double(SIGMA2),ncol=length(terms),nrow=length(terms))
  # OMEGAC
  ixstart__    <- aux_strFindAll(content__,'<omegac>')$end
  ixend__      <- aux_strFindAll(content__,'</omegac>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    omega_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    omega_text__ <- aux_strrep(omega_text__,'</row>','#')
    omega_text__ <- aux_strrep(omega_text__,'</col>','')
    omega_text__ <- aux_strrep(omega_text__,'<row rname=','')
    omega_text__ <- aux_strrep(omega_text__,'<col cname=','')
    omega_text__ <- aux_strrep(omega_text__,"'",'')
    omega_text__ <- gsub(x=omega_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(omega_text__,'#'), function(x) aux_strtrim(x)))
    OMEGAC <- matrix(NA,nrow=length(terms),ncol=length(terms))
    OMEGAC_names__  <- c()
    OMEGAC_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        OMEGAC[row,col] <- colterms__[col]
        OMEGAC[col,row] <- colterms__[col]
        OMEGAC_names__    <- c(OMEGAC_names__, sprintf('OMEGAC_%d,%d_',row,col))
        OMEGAC_values__   <- c(OMEGAC_values__, colterms__[col])
      }
    }
  } else {
    OMEGAC <- NULL
    OMEGAC_names__ <- NULL
    OMEGAC_values__ <- NULL
  }
  output__$OMEGAC$names   <- OMEGAC_names__
  output__$OMEGAC$values  <- as.double(OMEGAC_values__)
  names(output__$OMEGAC$names)   <- OMEGAC_names__
  names(output__$OMEGAC$values)  <- OMEGAC_names__
  output__$OMEGAC$matrix  <- matrix(as.double(OMEGAC),ncol=length(terms),nrow=length(terms))
  # SIGMAC
  ixstart__    <- aux_strFindAll(content__,'<sigmac>')$end
  ixend__      <- aux_strFindAll(content__,'</sigmac>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    sigma_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    sigma_text__ <- aux_strrep(sigma_text__,'</row>','#')
    sigma_text__ <- aux_strrep(sigma_text__,'</col>','')
    sigma_text__ <- aux_strrep(sigma_text__,'<row rname=','')
    sigma_text__ <- aux_strrep(sigma_text__,'<col cname=','')
    sigma_text__ <- aux_strrep(sigma_text__,"'",'')
    sigma_text__ <- gsub(x=sigma_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(sigma_text__,'#'), function(x) aux_strtrim(x)))
    SIGMAC <- matrix(NA,nrow=length(terms),ncol=length(terms))
    SIGMAC_names__  <- c()
    SIGMAC_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        SIGMAC[row,col] <- colterms__[col]
        SIGMAC[col,row] <- colterms__[col]
        SIGMAC_names__    <- c(SIGMAC_names__, sprintf('SIGMAC_%d,%d_',row,col))
        SIGMAC_values__   <- c(SIGMAC_values__, colterms__[col])
      }
    }
  } else {
    SIGMAC <- NULL
    SIGMAC_names__ <- NULL
    SIGMAC_values__ <- NULL
  }
  output__$SIGMAC$names   <- SIGMAC_names__
  output__$SIGMAC$values  <- as.double(SIGMAC_values__)
  names(output__$SIGMAC$names)   <- SIGMAC_names__
  names(output__$SIGMAC$values)  <- SIGMAC_names__
  output__$SIGMAC$matrix  <- matrix(as.double(SIGMAC),ncol=length(terms),nrow=length(terms))
  # THETASE
  ixstart__    <- aux_strFindAll(content__,'<thetase>')$end
  ixend__      <- aux_strFindAll(content__,'</thetase>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    thetase_text <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    thetase_text <- aux_strrep(thetase_text,'</val>','')
    thetase_text <- aux_strrep(thetase_text,sprintf("<val name='"),'')
    thetase_text <- gsub(x=thetase_text,pattern="[0-9]+'>","")
    thetase_text <- paste0("c(",aux_strrep(thetase_text,"\n",","),")")
    thetase_values <- eval(parse(text=thetase_text))
    thetase_names <- paste0("THETA",1:length(thetase_values))
  } else {
    thetase_values <- NULL
    thetase_names <- NULL
  }
  output__$THETA$standarderror <- thetase_values
  names(output__$THETA$standarderror) <- thetase_names
  # OMEGASE
  ixstart__    <- aux_strFindAll(content__,'<omegase>')$end
  ixend__      <- aux_strFindAll(content__,'</omegase>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    omega_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    omega_text__ <- aux_strrep(omega_text__,'</row>','#')
    omega_text__ <- aux_strrep(omega_text__,'</col>','')
    omega_text__ <- aux_strrep(omega_text__,'<row rname=','')
    omega_text__ <- aux_strrep(omega_text__,'<col cname=','')
    omega_text__ <- aux_strrep(omega_text__,"'",'')
    omega_text__ <- gsub(x=omega_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(omega_text__,'#'), function(x) aux_strtrim(x)))
    OMEGA2 <- matrix(NA,nrow=length(terms),ncol=length(terms))
    OMEGA2_names__  <- c()
    OMEGA2_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        OMEGA2[row,col] <- colterms__[col]
        OMEGA2[col,row] <- colterms__[col]
        OMEGA2_names__    <- c(OMEGA2_names__, sprintf('OMEGA2_%d,%d_',row,col))
        OMEGA2_values__   <- c(OMEGA2_values__, colterms__[col])
      }
    }
  } else {
    OMEGA2 <- NULL
    OMEGA2_names__ <- NULL
    OMEGA2_values__ <- NULL
  }
  output__$OMEGA2$standarderror          <- as.double(OMEGA2_values__)
  names(output__$OMEGA2$standarderror)   <- OMEGA2_names__
  output__$OMEGA2$standarderrorMATRIX    <- matrix(as.double(OMEGA2),ncol=length(terms),nrow=length(terms))
  # SIGMGASE
  ixstart__    <- aux_strFindAll(content__,'<sigmase>')$end
  ixend__      <- aux_strFindAll(content__,'</sigmase>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    sigma_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    sigma_text__ <- aux_strrep(sigma_text__,'</row>','#')
    sigma_text__ <- aux_strrep(sigma_text__,'</col>','')
    sigma_text__ <- aux_strrep(sigma_text__,'<row rname=','')
    sigma_text__ <- aux_strrep(sigma_text__,'<col cname=','')
    sigma_text__ <- aux_strrep(sigma_text__,"'",'')
    sigma_text__ <- gsub(x=sigma_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(sigma_text__,'#'), function(x) aux_strtrim(x)))
    SIGMA2 <- matrix(NA,nrow=length(terms),ncol=length(terms))
    SIGMA2_names__  <- c()
    SIGMA2_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        SIGMA2[row,col] <- colterms__[col]
        SIGMA2[col,row] <- colterms__[col]
        SIGMA2_names__    <- c(SIGMA2_names__, sprintf('SIGMA2_%d,%d_',row,col))
        SIGMA2_values__   <- c(SIGMA2_values__, colterms__[col])
      }
    }
  } else {
    SIGMA2 <- NULL
    SIGMA2_names__ <- NULL
    SIGMA2_values__ <- NULL
  }
  output__$SIGMA2$standarderror          <- as.double(SIGMA2_values__)
  names(output__$SIGMA2$standarderror)   <- SIGMA2_names__
  output__$SIGMA2$standarderrorMATRIX    <- matrix(as.double(SIGMA2),ncol=length(terms),nrow=length(terms))
  # OMEGACSE
  ixstart__    <- aux_strFindAll(content__,'<omegacse>')$end
  ixend__      <- aux_strFindAll(content__,'</omegacse>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    omega_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    omega_text__ <- aux_strrep(omega_text__,'</row>','#')
    omega_text__ <- aux_strrep(omega_text__,'</col>','')
    omega_text__ <- aux_strrep(omega_text__,'<row rname=','')
    omega_text__ <- aux_strrep(omega_text__,'<col cname=','')
    omega_text__ <- aux_strrep(omega_text__,"'",'')
    omega_text__ <- gsub(x=omega_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(omega_text__,'#'), function(x) aux_strtrim(x)))
    OMEGAC <- matrix(NA,nrow=length(terms),ncol=length(terms))
    OMEGAC_names__  <- c()
    OMEGAC_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        OMEGAC[row,col] <- colterms__[col]
        OMEGAC[col,row] <- colterms__[col]
        OMEGAC_names__    <- c(OMEGAC_names__, sprintf('OMEGAC_%d,%d_',row,col))
        OMEGAC_values__   <- c(OMEGAC_values__, colterms__[col])
      }
    }
  } else {
    OMEGAC <- NULL
    OMEGAC_names__ <- NULL
    OMEGAC_values__ <- NULL
  }
  output__$OMEGAC$standarderror         <- as.double(OMEGAC_values__) * as.double(output__$OMEGA2$standarderror!=0)
  names(output__$OMEGAC$standarderror)  <- OMEGAC_names__
  output__$OMEGAC$standarderrorMATRIX   <- matrix(as.double(OMEGAC) * as.double(output__$OMEGA2$standarderrorMATRIX!=0),ncol=length(terms),nrow=length(terms))
  # SIGMACSE
  ixstart__    <- aux_strFindAll(content__,'<sigmacse>')$end
  ixend__      <- aux_strFindAll(content__,'</sigmacse>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    sigma_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    sigma_text__ <- aux_strrep(sigma_text__,'</row>','#')
    sigma_text__ <- aux_strrep(sigma_text__,'</col>','')
    sigma_text__ <- aux_strrep(sigma_text__,'<row rname=','')
    sigma_text__ <- aux_strrep(sigma_text__,'<col cname=','')
    sigma_text__ <- aux_strrep(sigma_text__,"'",'')
    sigma_text__ <- gsub(x=sigma_text__,pattern='[0-9]+>','')
    terms <- unlist(lapply(aux_explode(sigma_text__,'#'), function(x) aux_strtrim(x)))
    SIGMAC <- matrix(NA,nrow=length(terms),ncol=length(terms))
    SIGMAC_names__  <- c()
    SIGMAC_values__  <- c()
    for (row in seq_along(terms)) {
      rowtext__ <- aux_strtrim(terms[row])
      colterms__ <- aux_explode(rowtext__,"\n")
      for (col in seq_along(colterms__)) {
        SIGMAC[row,col] <- colterms__[col]
        SIGMAC[col,row] <- colterms__[col]
        SIGMAC_names__    <- c(SIGMAC_names__, sprintf('SIGMAC_%d,%d_',row,col))
        SIGMAC_values__   <- c(SIGMAC_values__, colterms__[col])
      }
    }
  } else {
    SIGMAC <- NULL
    SIGMAC_names__ <- NULL
    SIGMAC_values__ <- NULL
  }
  output__$SIGMAC$standarderror         <- as.double(SIGMAC_values__) * as.double(output__$SIGMA2$standarderror!=0)
  names(output__$SIGMAC$standarderror)  <- SIGMAC_names__
  output__$SIGMAC$standarderrorMATRIX   <- matrix(as.double(SIGMAC) * as.double(output__$SIGMA2$standarderrorMATRIX!=0),ncol=length(terms),nrow=length(terms))
  # Covariance
  ixstart__    <- aux_strFindAll(content__,'<covariance>')$end
  ixend__      <- aux_strFindAll(content__,'</covariance>')$start
  if (length(ixstart__) != length(ixend__)) {
    warning("Problems with covariance estimation in NONMEM")
    output__$COVARIANCE_MATRIX <- NULL
  } else {
    ixstart__  <- ixstart__[length(ixstart__)]
    ixend__    <- ixend__[length(ixend__)]
    if (!is.null(ixstart__) & !is.null(ixend__)) {
      cormatrix_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
      cormatrix_text__ <- aux_strrep(cormatrix_text__,"'",'')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'</row>','#')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'</col>','')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'<col cname=','')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'<','')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'>',' ')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'(','_')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,')','_')
      m <- gregexpr(text=cormatrix_text__,pattern='row rname=([A-Z_,0-9]+)')
      cormatrix_names__ <- regmatches(x=cormatrix_text__,m=m)
      cormatrix_names__ <- gsub(x=cormatrix_names__[[1]],pattern="row rname=","")
      cormatrix_text__ <- gsub(x=cormatrix_text__,pattern='row rname=([A-Z_,0-9]+)',replacement='')
      cormatrix_text__ <- gsub(x=cormatrix_text__,pattern='[A-Z_,0-9]+ ',replacement='')
      terms <- aux_explode(cormatrix_text__,'#')
      terms <- unlist(lapply(terms, function(x) aux_strtrim(x)))
      COV_MATRIX__ <- matrix(NA,nrow=length(terms),ncol=length(terms))
      for (row in seq_along(terms)) {
        rowtext__ <- aux_strtrim(terms[row])
        colterms__ <- aux_explode(rowtext__,"\n")
        for (col in seq_along(colterms__)) {
          value <- colterms__[col]
          COV_MATRIX__[row,col] <- colterms__[col]
          COV_MATRIX__[col,row] <- colterms__[col]
        }
      }
    } else {
      COV_MATRIX__ <- NULL
      cormatrix_names__ <- NULL
    }
    output__$COVARIANCE_MATRIX <- matrix(as.double(COV_MATRIX__),ncol=length(terms),nrow=length(terms))
    rownames(output__$COVARIANCE_MATRIX) <- cormatrix_names__
    colnames(output__$COVARIANCE_MATRIX) <- cormatrix_names__
  }
  ixstart__    <- aux_strFindAll(content__,'<correlation>')$end
  ixend__      <- aux_strFindAll(content__,'</correlation>')$start
  if (length(ixstart__) != length(ixend__)) {
    warning("Problems with correlation matrix estimation in NONMEM")
    output__$CORRELATION_MATRIX <- NULL
  } else {
    ixstart__  <- ixstart__[length(ixstart__)]
    ixend__    <- ixend__[length(ixend__)]
    if (!is.null(ixstart__) & !is.null(ixend__)) {
      cormatrix_text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
      cormatrix_text__ <- aux_strrep(cormatrix_text__,"'",'')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'</row>','#')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'</col>','')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'<col cname=','')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'<','')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'>',' ')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,'(','_')
      cormatrix_text__ <- aux_strrep(cormatrix_text__,')','_')
      m <- gregexpr(text=cormatrix_text__,pattern='row rname=([A-Z_,0-9]+)')
      cormatrix_names__ <- regmatches(x=cormatrix_text__,m=m)
      cormatrix_names__ <- gsub(x=cormatrix_names__[[1]],pattern="row rname=","")
      cormatrix_text__ <- gsub(x=cormatrix_text__,pattern='row rname=([A-Z_,0-9]+)',replacement='')
      cormatrix_text__ <- gsub(x=cormatrix_text__,pattern='[A-Z_,0-9]+ ',replacement='')
      terms <- aux_explode(cormatrix_text__,'#')
      terms <- unlist(lapply(terms, function(x) aux_strtrim(x)))
      COR_MATRIX__ <- matrix(NA,nrow=length(terms),ncol=length(terms))
      for (row in seq_along(terms)) {
        rowtext__ <- aux_strtrim(terms[row])
        colterms__ <- aux_explode(rowtext__,"\n")
        for (col in seq_along(colterms__)) {
          value <- colterms__[col]
          COR_MATRIX__[row,col] <- colterms__[col]
          COR_MATRIX__[col,row] <- colterms__[col]
        }
      }
    } else {
      COR_MATRIX__ <- NULL
      cormatrix_names__ <- NULL
    }
    output__$CORRELATION_MATRIX <- matrix(as.double(COR_MATRIX__),ncol=length(terms),nrow=length(terms))
    rownames(output__$CORRELATION_MATRIX) <- cormatrix_names__
    colnames(output__$CORRELATION_MATRIX) <- cormatrix_names__
  }
  output__$COV_COR_MATRIX_NAMES <- cormatrix_names__

  # Get estimated information by checking 1e10 values in standard errors
  output__$THETA$estimated <- as.numeric(!output__$THETA$standarderror %in% c(0,1e10))
  output__$OMEGA2$estimated <- as.numeric(!output__$OMEGA2$standarderror %in% c(0,1e10))
  output__$OMEGAC$estimated <- as.numeric(!output__$OMEGAC$standarderror %in% c(0,1e10))
  output__$SIGMA2$estimated <- as.numeric(!output__$SIGMA2$standarderror %in% c(0,1e10))
  output__$SIGMAC$estimated <- as.numeric(!output__$SIGMAC$standarderror %in% c(0,1e10))

  output__$OMEGA2$estimatedMATRIX <- matrix(as.numeric(!output__$OMEGA2$standarderrorMATRIX %in% c(0,1e10)),ncol=ncol(output__$OMEGA2$standarderrorMATRIX))
  output__$OMEGAC$estimatedMATRIX <- matrix(as.numeric(!output__$OMEGAC$standarderrorMATRIX %in% c(0,1e10)),ncol=ncol(output__$OMEGAC$standarderrorMATRIX))
  output__$SIGMA2$estimatedMATRIX <- matrix(as.numeric(!output__$SIGMA2$standarderrorMATRIX %in% c(0,1e10)),ncol=ncol(output__$SIGMA2$standarderrorMATRIX))
  output__$SIGMAC$estimatedMATRIX <- matrix(as.numeric(!output__$SIGMAC$standarderrorMATRIX %in% c(0,1e10)),ncol=ncol(output__$SIGMAC$standarderrorMATRIX))

  # Determine the number of estimated parameters
  output__$NRPARAM_ESTIMATED <- sum(output__$THETA$estimated) + sum(output__$OMEGAC$estimated) + sum(output__$SIGMAC$estimated)

  # Parse shrinkage
  ixstart__    <- aux_strFindAll(content__,'<etashrinksd>')$end
  ixend__      <- aux_strFindAll(content__,'</etashrinksd>')$start
  ixstart__  <- ixstart__[length(ixstart__)]
  ixend__    <- ixend__[length(ixend__)]
  if (!is.null(ixstart__)) {
    text__ <- aux_strtrim(substr(content__,ixstart__+1,ixend__-1))
    text__ <- gsub("<row rname='[^\n]+'>\n","",text__)
    text__ <- gsub("</row>","",text__)
    text__ <- gsub("<col cname='[^\n]+'>","",text__)
    text__ <- gsub("</col>","",text__)
    text__ <- gsub("'>","=",text__)
    text__ <- aux_strtrim(text__)
    values <- as.numeric(aux_explode(text__,"\n"))
  } else {
    values <- NULL
  }
  output__$ETA_SHRINKAGE <- values
  return(output__)
}

# Parse the model annotation and allow for default values if issues
parseNONMEMprojectHeader <- function (NMcontrolFile,NTHETA__,NETA__,NSIGMA__) {

  # Initialize default output list
  out__ <- list(
    THETANAMES          = paste0("THETA",1:NTHETA__),
    THETACOVS           = rep(0,NTHETA__),
    THETAERRS           = rep(0,NTHETA__),
    THETADESCRIPTION    = rep("Undefined",NTHETA__),
    THETAUNIT           = rep("Undefined",NTHETA__),
    THETATRANS          = rep("N",NTHETA__),
    ETANAMES            = paste0("ETA",1:NETA__),
    ETADESCRIPTION      = rep("Undefined",NETA__),
    SIGMANAMES          = paste0("SIGMA",1:NSIGMA__),
    SIGMADESCRIPTION    = rep("Undefined",NSIGMA__),
    SIGMAUNIT           = rep("Undefined",NSIGMA__)
  )

  # Attempt to get model annotation
  model__ <- aux_fileread(NMcontrolFile,collapserows = FALSE)
  ixstart__ <- which(grepl("MODEL ANNOTATION START",model__))
  ixend__ <- which(grepl("MODEL ANNOTATION END",model__))
  if (length(ixstart__) == 0 | length(ixend__) == 0) {
    message('Model annotation could not be found in NONMEM control file - using default settings.')
    return(out__)
  }
  if (length(ixstart__) > 1 | length(ixend__) > 1) {
    message('Model annotation identifiers for start and/or end appear several times - using default settings.')
    return(out__)
  }

  # Get annotation lines
  lines__ <- model__[(ixstart__+1):(ixend__-1)]

  # Remove first ";" and trim
  lines__ <- gsub("^;[ ]*","###",lines__)

  # Parse THETA information
  for (k__ in 1:NTHETA__) {
    name__ <- paste0("THETA",k__)
    ix__ <- which(grepl(paste0("###",name__,":"),lines__))
    if (length(ix__) > 1) {
      message(paste0("Information for ",name__," provided more than once. Only using first occurrence"))
      ix__ <- ix__[1]
    }
    if (length(ix__) == 0) {
      message(paste0("Information for ",name__," not provided. Using default settings"))
    } else {
      line__ <- aux_strtrim(gsub(paste0("###",name__,":"),"",lines__[ix__]))
      terms__ <- aux_strtrim(aux_explode(line__))
      if (length(terms__) < 5) {
        message(paste0("Information for ",name__," does not contain all required information. Using default settings"))
      } else {
        out__$THETANAMES[k__] <- terms__[1]
        out__$THETACOVS[k__] <- terms__[2]
        out__$THETAERRS[k__] <- terms__[3]
        out__$THETAUNIT[k__] <- terms__[4]
        out__$THETADESCRIPTION[k__] <- terms__[5]
        if (length(terms__) == 6) out__$THETATRANS[k__] <- terms__[6]

      }
    }
  }

  # Parse ETA information
  for (k__ in 1:NETA__) {
    name__ <- paste0("ETA",k__)
    ix__ <- which(grepl(paste0("###",name__,":"),lines__))
    if (length(ix__) > 1) {
      message(paste0("Information for ",name__," provided more than once. Only using first occurrence"))
      ix__ <- ix__[1]
    }
    if (length(ix__) == 0) {
      message(paste0("Information for ",name__," not provided. Using default settings"))
    } else {
      line__ <- aux_strtrim(gsub(paste0("###",name__,":"),"",lines__[ix__]))
      terms__ <- aux_strtrim(aux_explode(line__))
      if (length(terms__) != 2) {
        message(paste0("Information for ",name__," does not contain all required information. Using default settings"))
      } else {
        out__$ETANAMES[k__] <- terms__[1]
        out__$ETADESCRIPTION[k__] <- terms__[2]
      }
    }
  }

  # Parse SIGMA information
  for (k__ in 1:NSIGMA__) {
    name__ <- paste0("SIGMA",k__)
    ix__ <- which(grepl(paste0("###",name__,":"),lines__))
    if (length(ix__) > 1) {
      message(paste0("Information for ",name__," provided more than once. Only using first occurrence"))
      ix__ <- ix__[1]
    }
    if (length(ix__) == 0) {
      message(paste0("Information for ",name__," not provided. Using default settings"))
    } else {
      line__ <- aux_strtrim(gsub(paste0("###",name__,":"),"",lines__[ix__]))
      terms__ <- aux_strtrim(aux_explodePC(line__))
      if (length(terms__) != 3) {
        message(paste0("Information for ",name__," does not contain all required information. Using default settings"))
      } else {
        out__$SIGMANAMES[k__] <- terms__[1]
        out__$SIGMAUNIT[k__] <- terms__[2]
        out__$SIGMADESCRIPTION[k__] <- terms__[3]
      }
    }
  }

  # Done and return result
  return(out__)
}

# Calculate AIC and BIC
calcAICBIC <- function(OFV,nData,nParam) {
  AIC <- as.numeric(OFV) + 2*as.numeric(nParam)
  BIC <- as.numeric(OFV) + log(as.numeric(nData))*as.numeric(nParam)
  list(AIC=AIC,BIC=BIC)
}
