### Overloading "+" to combine IQRrmd objects ----
###' Concatenate IQRrmd objects
#'
#' @param a Character string or number or IQRrmd object
#' @param b Character string or number or IQRrmd object
#' @return Concatenated IQRrmd object "ab"
#' @export
#' @family IQR report
"+.IQRrmd" <- function (a,b) {
  out__ <- paste0(a,b)
  class(out__) <- "IQRrmd"
  return(out__)
}

### Overloading "print" for IQRrmd objects ----
###' Overloading "print" for IQRrmd objects
#'
#' @param x IQRrmd object
#' @param ... Additional parameters
#' @export
#' @family IQR report
print.IQRrmd <- function (x, ...) {
  cat(x)
  cat("\n\nIQRrmd object")
}

### Export IQRrmd object to an rmd file ----
###' Export IQRrmd object to an rmd file
#'
#' @param rmdDoc IQRrmd object
#' @param filename Filename (might include path). If not already appended, ".rmd"
#'    will be appended.
#' @export
#' @family IQR report
export_IQRrmd <- function (rmdDoc, filename) {
  filename <- paste0(aux_strrep(filename,".rmd",""),".rmd")
  aux_filewrite(paste0(rmdDoc),filename)
}

### Create empty IQRrmd object ----
###' Create empty IQRrmd object
#'
#' @return Empty IQRrmd object
#' @export
#' @family IQR report
rmdEMPTY <- function() {
  out__ <- ""
  class(out__) <- "IQRrmd"
  return (out__)
}

### Line break ----
###' Add a simple line break
#'
#' @return Linebreak RMD object
#' @export
#' @family IQR report
rmdLINEBREAK <- function() {
  out__ <- "\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### New parapgraph ----
###' Start a new paragraph
#'
#' @return New paragraph RMD object
#' @export
#' @family IQR report
rmdPARAGRAPH <- function() {
  out__ <- "\n\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### Create empty IQRrmd object ----
###' Create empty IQRrmd object
#'
#' @return Empty IQRrmd object
#' @export
#' @family IQR report
rmdEMPTY <- function() {
  out__ <- ""
  class(out__) <- "IQRrmd"
  return (out__)
}
### Titlepage command ----
###' Generate title RMD command
#'
#' In the title section of an RMD document the template to be used and the
#' title page information to be shown are defined. IQRtools works with IQReport and
#' the DefaultStyle.rmdt template document. If you want to use a different template
#' you can do that but you would need to have one. Contact info@intiquan.com to
#' learn more. You basically can get IQRtools generate Word reports in the desired
#' style, required for your organization.
#'
#' @param template Name of the *.rmdt template file to be used
#' @param title Character string with title of the document
#' @param subtitle Character string with subtitle of the document
#' @param date Character string with date to be shown
#' @return Title IQRrmd object
#' @export
#' @family IQR report
rmdTITLE <- function(template="DefaultStyle.rmdt",
                     title="Default Title",
                     subtitle="Default Subtitle",
                     date=format(Sys.Date(),"%d-%b-%Y")) {

  out__ <-              "=====\n"
  out__ <- paste0(out__,"TITLE:    ",title,"\n")
  out__ <- paste0(out__,"SUBTITLE: ",subtitle,"\n")
  out__ <- paste0(out__,"DATE:     ",date,"\n")
  out__ <- paste0(out__,"TEMPLATE: ",template,"\n")
  out__ <- paste0(out__,"=====\n\n")

  class(out__) <- "IQRrmd"
  return(out__)
}

### NEWPAGE command ----
###' Generate newpage RMD command
#'
#' @return New page command as IQRrmd object
#' @export
#' @family IQR report
rmdNEWPAGE <- function() {
  out__ <- "\n!NEWPAGE\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### APPROVALS command ----
###' Add an approvals page to the report (after title)
#'
#' This command is only working as expected if the used template actually has an
#' approvals page included. Otherwise this command will be ignored.
#'
#' @return APPROVALS command as IQRrmd object
#' @export
#' @family IQR report
rmdAPPROVALS <- function() {
  out__ <- "\n!APPROVALS\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### NOINTRO command ----
###' Do not include TOC/TOF/TOT in DOCX  report
#'
#' The !NOINTRO command will remove the table of contents, figures, and tables
#' from the Word document.
#'
#' @return NOINTRO command as IQRrmd object
#' @export
#' @family IQR report
rmdNOINTRO <- function() {
  out__ <- "\n!NOINTRO\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### LANDSCAPE command ----
###' Switch to Landscape mode
#'
#' This will only work if the section where the landcape command is called contains
#' some text ... tables or figures do not count ...
#'
#' @return Landscape command as IQRrmd object
#' @export
#' @family IQR report
rmdLANDSCAPE <- function() {
  out__ <- "\n!LANDSCAPE\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### PORTRAIT command ----
###' Switch to Portrait mode
#'
#' @return Portrait command as IQRrmd object
#' @export
#' @family IQR report
rmdPORTRAIT <- function() {
  out__ <- "\n!PORTRAIT\n"
  class(out__) <- "IQRrmd"
  return (out__)
}

### Section command ----
###' Generate Section RMD command
#'
#' Allows generation of RMD strings to implement sections (level=1), subsections
#' (level=2), subsubsections (level=3), etc. Sections can be numbered (numbered=TRUE)
#' or unnumbered (numbered=FALSE). A label can be given to be able to crossreference
#' this section.
#'
#' Unnumbered sections have the limitation to level=1 and only a single unnumbered section
#' can be present in a document - the first one.
#'
#' @param title Character string with title of section
#' @param label Character string (no spaces) with label for cross-reference use.
#'  Limited to 40 characters.
#' @param level Integer value (1,2,3,etc.) to indicate the section level
#' @param numbered Boolean to define if section shoud be numbered or not (experimental)
#' @return Section RMD command as IQRrmd object
#' @export
#' @family IQR report
rmdSECTION <- function(title="Section Title",label=NULL,level=1,numbered=TRUE) {
  out__ <- paste0("\n\n",paste0(rep("#",level),collapse=""))
  if (!numbered)
    out__ <- paste0(out__,"*")
  out__ <- paste0(out__," ",title)
  if (!is.null(label))
    out__ <- paste0(out__," {#",label,"}")
  out__ <- paste0(out__,"\n")
  class(out__) <- "IQRrmd"
  return(out__)
}

### !URL RMD command ----
###' Generate RMD command to include a URL
#'
#' @param url Character string, specifying the URL
#' @param caption Text to display as hyperlink
#' @return URL command as IQRrmd object
#' @export
#' @family IQR report
rmdURL <- function(url="http://www.intiquan.com",caption=NULL) {
  out__ <- "!URL"
  if (!is.null(caption))
    out__ <- paste0(out__,"[",caption,"]")
  out__ <- paste0(out__,"(",url,")")
  class(out__) <- "IQRrmd"
  return (out__)
}

### !COM RMD command ----
###' Generate RMD command to include a Word comment
#'
#' @param comment Character string, specifying the comment text. Line breaks are
#'   allowed.
#' @param author Character string, specifying the author
#' @return COM command as IQRrmd object
#' @export
#' @family IQR report
rmdCOMMENT <- function(comment="some comment",author=NULL) {
  out__ <- "!COM"
  if (!is.null(author))
    out__ <- paste0(out__,"[author:",author,"]")
  out__ <- paste0(out__,"{",comment,"}")
  class(out__) <- "IQRrmd"
  return (out__)
}

### !TXT RMD command ----
###' Generate RMD command to include a text from a file
#'
#' @param file Character string, specifying the absolute or relative path to the
#' text file to include.
#' @param caption Caption for the text file. If NULL then no caption is shown.
#' @param fontsize Integer value for the fontsize.
#' @param style Word DOCX template style to use for the text (default: "CODE")
#' @param label Character string to be used for cross-referencing. If label and caption
#'   provided, then the caption will be numbered.
#' @return TXT command as IQRrmd object
#' @export
#' @family IQR report
rmdTXT <- function(file,caption=NULL,fontsize=10,style="CODE",label=NULL) {
  out__ <- "\n!TXT"
  if (!is.null(c(caption,style,label))) {
    opttext__ <- ""
    if (!is.null(caption))
      opttext__ <- paste0(opttext__,"caption:",caption,",")
    if (!is.null(style))
      opttext__ <- paste0(opttext__,"style:",style,"(",fontsize,"),")
    if (!is.null(label))
      opttext__ <- paste0(opttext__,"label:",label,",")
    if (substr(opttext__,nchar(opttext__),nchar(opttext__))==",") opttext__ <- substr(opttext__,1,nchar(opttext__)-1)
    out__ <- paste0(out__,"[",opttext__,"]")
  }
  out__ <- paste0(out__,"(",file,")\n")
  class(out__) <- "IQRrmd"
  return (out__)
}

### Table function ----
###' Generate RMD command to include a table from a file
#'
#' Allows to generate RMD code to include a table defined in a text file.
#' To include a table from an R dataframe, please use the function
#' rmdTABLEDF.
#'
#' @param file Character string, specifying the absolute or relative path to the
#'   table text file to include.
#' @param caption Caption for the table. If NULL then the title information in the
#'   table text file is used as caption.
#' @param fontsize Integer value for the fontsize.
#' @param label Character string to be used for cross-referencing.
#' @param ignoreCaption Caption can be ignored if desired (no caption will be shown for table)
#' @param valueTable Defines is table can be used as a "Value Table", allowing to reference
#'   values in the table. A value table requires the presence of a "value" and a "label" column.
#'   If these are not present, an error will occur.
# @return TAB command as IQRrmd object
#' @export
#' @family IQR report
rmdTABLE <- function(file,caption=NULL,fontsize=10,label=NULL,
                     ignoreCaption=FALSE,
                     valueTable=FALSE) {
  out__ <- "\n!TAB"
  if (!is.null(c(caption,fontsize,label,ignoreCaption,valueTable))) {
    opttext__ <- ""
    if (!is.null(caption))
      opttext__ <- paste0(opttext__,"caption:",caption,",")
    if (!is.null(fontsize))
      opttext__ <- paste0(opttext__,"size:",fontsize,",")
    if (!is.null(label))
      opttext__ <- paste0(opttext__,"label:",label,",")
    if (ignoreCaption) {
      opttext__ <- paste0(opttext__,"ignoreCaption:true,")
    } else {
      opttext__ <- paste0(opttext__,"ignoreCaption:false,")
    }
    if (valueTable) {
      opttext__ <- paste0(opttext__,"valueTable:true,")
    } else {
      opttext__ <- paste0(opttext__,"valueTable:false,")
    }

    if (substr(opttext__,nchar(opttext__),nchar(opttext__))==",") opttext__ <- substr(opttext__,1,nchar(opttext__)-1)
    out__ <- paste0(out__,"[",opttext__,"]")
  }
  out__ <- paste0(out__,"(",file,")\n")
  class(out__) <- "IQRrmd"
  return (out__)
}

### rmdTABLEDF function ----
###' Generate RMD command to include a table from a dataframe
#'
#' Allows to generate RMD code to include a table defined by a dataframe.
#' To include a table from a table text file, please use the function
#' rmdTABLE.
#'
#' @param df Dataframe to export as table in RMD.
#' @param caption Caption for the table.
#' @param fontsize Integer value for the fontsize.
#' @param label Character string to be used for cross-referencing.
#' @param ignoreCaption Caption can be ignored if desired (no caption will be shown for table)
#' @param valueTable Defines is table can be used as a "Value Table", allowing to reference
#'   values in the table. A value table requires the presence of a "value" and a "label" column.
#'   If these are not present, an error will occur.
#' @param footertext Defined text for the table footer.
#' @return Inline table command as IQRrmd object
#' @export
#' @family IQR report
rmdTABLEDF <- function(df,caption=NULL,fontsize=10,label=NULL,ignoreCaption=FALSE,valueTable=FALSE,footertext=NULL) {
  out__ <- "\n!TABINLINE["
  if (!is.null(caption))
    out__ <- paste0(out__,"caption:",caption,",")
  if (!is.null(fontsize))
    out__ <- paste0(out__,"size:",fontsize,",")
  if (!is.null(label))
    out__ <- paste0(out__,"label:",label,",")
  if (ignoreCaption) {
    out__ <- paste0(out__,"ignoreCaption:true,")
  } else {
    out__ <- paste0(out__,"ignoreCaption:false,")
  }
  if (valueTable) {
    out__ <- paste0(out__,"valueTable:true")
  } else {
    out__ <- paste0(out__,"valueTable:false")
  }
  out__ <- paste0(out__,"]")
  out__

  ##############################-
  # Convert dataframe to MD ----
  ##############################-
  headerNames__ <- names(df)

  # Convert df entries to char
  # Handle <NA> by replacing with "NA"
  # Also remove "\n" and "\r"
  dfchar__ <- data.frame(
    lapply(df, function (x__) {
      out__ <- as.character(x__)
      out__[is.na(out__)] <- "NA"
      out__ <- gsub(pattern="\n",replacement="",x=out__)
      out__ <- gsub(pattern="\r",replacement="",x=out__)
      out__
    })
    ,stringsAsFactors=FALSE
  )

  # Get max char lengths in columns - including header
  ncharCol__ <- sapply(1:ncol(dfchar__), function (y__) max(nchar(headerNames__[y__]),max(nchar(dfchar__[[y__]]))))

  # Construct table MD text - header
  headerExp__ <- sapply(seq_along(headerNames__), function(k__) aux_postFillChar(value2postfill=headerNames__[k__],lengthString=ncharCol__[k__],fillChar=" ") )
  headerLine__ <- sapply(headerExp__, function (x__) paste0(rep("-",nchar(x__)),collapse=""))

  headerOut__ <- paste0("| ",
                        paste(headerExp__,collapse = " | "),
                        " |\n",
                        "| ",
                        paste(headerLine__,collapse = " | "),
                        " |\n")

  # Construct table MD - content
  textContent__ <- paste0(sapply(1:nrow(dfchar__), function (nrow__) {
    textExp__ <- sapply(seq_along(headerNames__), function(k__) aux_postFillChar(value2postfill=dfchar__[nrow__,k__],lengthString=ncharCol__[k__],fillChar=" ") )
    textOut__ <- paste0("| ",
                        paste(textExp__,collapse = " | "),
                        " |\n")
  }),collapse="")

  # Combine
  tableOut__ <- paste0("\n",headerOut__,textContent__)

  # Add footertext if present
  if (!is.null(footertext)) {
    footertext <- gsub("\n","<br>",footertext)
    tableOut__ <- paste0(tableOut__,"{",footertext,"}")
  }

  out__ <- paste0(out__,tableOut__)

  class(out__) <- "IQRrmd"
  return (out__)
}

### Figure function ----
###' Generate RMD command to include a figure from a file
#'
#' Allows to generate RMD code to include a figure defined in a pdf, png, jpg file.
#' From multi page PDFs selected pages or page ranges can be included.
#'
#' @param file Character string, specifying the absolute or relative path to the
#'   figure file to include (pdf, png, jpg, etc.).
#' @param caption Caption for the table. If NULL then the title information in the
#'   table text file is used as caption.
#' @param label Character string to be used for cross-referencing.
#' @param legend Character string with text that is written as "legend" under the figure.
#' @param pages A character string with information about which pages of a multi-page PDF
#'   should be included into the report. If undefined all pages will be exported with the same
#'   caption and figure number. The legend will be repeated after each figure if argument
#'   \code{repeatLegend} is set to TRUE.
#'   Example: pages="3 5-8" exports page 3,5,6,7,and 8.
#' @param scale Number value (percent) for scaling the figure.
#' @param crop Boolean. If set to TRUE then figure will be cropped to remove surrounding white space.
#' @param repeatLegend See \code{legend}.
#' @param ignoreCaption Caption can be ignored if desired (no caption will be shown for figure)
#' @return FIG command as IQRrmd object
#' @export
#' @family IQR report


rmdFIGURE <- function(file,
                      caption=NULL,label=NULL,
                      legend=NULL,
                      pages=NULL,
                      scale=100, crop=TRUE,
                      repeatLegend=TRUE,
                      ignoreCaption=FALSE) {
  out__ <- "\n!FIG"
  if (!is.null(c(caption,label,legend,pages,scale,crop,repeatLegend,ignoreCaption))) {
    opttext__ <- ""
    if (!is.null(caption))
      opttext__ <- paste0(opttext__,"caption:",caption,",")
    if (!is.null(label))
      opttext__ <- paste0(opttext__,"label:",label,",")
    if (!is.null(pages))
      opttext__ <- paste0(opttext__,"pages:",pages,",")
    if (!is.null(scale))
      opttext__ <- paste0(opttext__,"scale:",scale,",")
    if (crop) {
      opttext__ <- paste0(opttext__,"crop:true,")
    } else {
      opttext__ <- paste0(opttext__,"crop:false,")
    }
    if (ignoreCaption) {
      opttext__ <- paste0(opttext__,"ignoreCaption:true,")
    } else {
      opttext__ <- paste0(opttext__,"ignoreCaption:false,")
    }
    if (repeatLegend) {
      opttext__ <- paste0(opttext__,"repeatLegend:true,")
    } else {
      opttext__ <- paste0(opttext__,"repeatLegend:false,")
    }
    if (substr(opttext__,nchar(opttext__),nchar(opttext__))==",") opttext__ <- substr(opttext__,1,nchar(opttext__)-1)
    out__ <- paste0(out__,"[",opttext__,"]")
  }
  # Add filepath
  out__ <- paste0(out__,"(",file,")")
  # Add legend
  if (!is.null(legend))
    out__ <- paste0(out__,"\n{\n",gsub("\n","  \n",legend),"\n}")
  # Add line break
  out__ <- paste0(out__,"\n")
  # Make IQRrmd object
  class(out__) <- "IQRrmd"
  return (out__)
}

### PDF include function ----
###' Generate RMD command to include a PDF from a file
#'
#' The main difference to rmdFIGURE is that rmdPDF does not create a caption and
#' not an entry into the List of Figures. This allows to include PDF documents as
#' sections ... if desired. Compliance text is not shown.
#'
#' @param file Character string, specifying the absolute or relative path to the
#'   PDF file to include.
#' @param pages A character string with information about which pages of a multi-page PDF
#'   should be included into the report.
#'   Example: pages="3 5-8" exports page 3,5,6,7,and 8.
#' @param scale Number value (percent) for scaling the figure.
#' @param crop Boolean. If set to TRUE then figure will be cropped to remove surrounding white space.
#' @return PDF command as IQRrmd object
#' @export
#' @family IQR report
rmdPDF <- function(file,pages=NULL,scale=100,crop=TRUE) {
  out__ <- "\n!PDF"
  if (!is.null(c(pages,scale,crop))) {
    opttext__ <- ""
    if (!is.null(pages))
      opttext__ <- paste0(opttext__,"pages:",pages,",")
    if (!is.null(scale))
      opttext__ <- paste0(opttext__,"scale:",scale,",")
    if (crop) {
      opttext__ <- paste0(opttext__,"crop:true,")
    } else {
      opttext__ <- paste0(opttext__,"crop:false,")
    }
    if (substr(opttext__,nchar(opttext__),nchar(opttext__))==",") opttext__ <- substr(opttext__,1,nchar(opttext__)-1)
    out__ <- paste0(out__,"[",opttext__,"]")
  }
  # Add filepath
  out__ <- paste0(out__,"(",file,")")
  # Add line break
  out__ <- paste0(out__,"\n")
  # Make IQRrmd object
  class(out__) <- "IQRrmd"
  return (out__)
}

### Section Cross reference ----
###' Section Cross reference
#'
#' Allows to reference a section by the section label.
#' "@SEC[caption](label)"
#'
#' @param label Character string, specifying the label of the section. Labels are limited to
#'   40 characters and should not include whitespaces.
#' @param caption A character string with alernative text to show (instead of "Section x.y.z").
#' @return Section cross reference command as IQRrmd object
#' @export
#' @family IQR report
rmdSECref <- function(label,caption=NULL) {
  out__ <- " @SEC"
  if (!is.null(caption))
    out__ <- paste0(out__,"[",caption,"]")
  out__ <- paste0(out__,"(",label,")")
  class(out__) <- "IQRrmd"
  return (out__)
}

### Text Cross reference ----
###' Text Cross reference
#'
#' Allows to reference an included text by label.
#' "@TXT[caption](label)"
#'
#' If caption ignored the cross-reference will not work.
#'
#' @param label Character string, specifying the label of the text. Labels are limited to
#'   40 characters and should not include whitespaces.
#' @param caption A character string with alernative text to show (instead of "Text x").
#' @return Text cross reference command as IQRrmd object
#' @export
#' @family IQR report
rmdTXTref <- function(label,caption=NULL) {
  out__ <- " @TXT"
  if (!is.null(caption))
    out__ <- paste0(out__,"[",caption,"]")
  out__ <- paste0(out__,"(",label,")")
  class(out__) <- "IQRrmd"
  return (out__)
}

### Figure Cross reference ----
###' Figure Cross reference
#'
#' Allows to reference an included figure by label.
#' "@FIG[caption](label)"
#'
#' If caption ignored the cross-reference will not work.
#'
#' @param label Character string, specifying the label of the figure. Labels are limited to
#'   40 characters and should not include whitespaces.
#' @param caption A character string with alernative text to show (instead of "Text x").
#' @return Figure cross reference command as IQRrmd object
#' @export
#' @family IQR report
rmdFIGref <- function(label,caption=NULL) {
  out__ <- " @FIG"
  if (!is.null(caption))
    out__ <- paste0(out__,"[",caption,"]")
  out__ <- paste0(out__,"(",label,")")
  class(out__) <- "IQRrmd"
  return (out__)
}

### Table Cross reference ----
###' Table Cross reference
#'
#' Allows to reference an included table by label.
#' "@TAB[caption](label)"
#'
#' If caption ignored the cross-reference will not work.
#'
#' @param label Character string, specifying the label of the table. Labels are limited to
#'   40 characters and should not include whitespaces.
#' @param caption A character string with alernative text to show (instead of "Text x").
#' @return Table cross reference command as IQRrmd object
#' @export
#' @family IQR report
rmdTABref <- function(label,caption=NULL) {
  out__ <- " @TAB"
  if (!is.null(caption))
    out__ <- paste0(out__,"[",caption,"]")
  out__ <- paste0(out__,"(",label,") ")
  class(out__) <- "IQRrmd"
  return (out__)
}

### Value Table cross reference ----
###' Value Table cross reference
#'
#' Allows to refence (display) values that are stored in labeled tables.
#' Two ways are possible:
#'  * Either by referncing row (not counting header row) and column number:
#'    "@VAL[row,col](labelTable)"
#'  * Or by referencing the label in a ValueTable:
#'    "@VAL[labelValue](labelTable)"
#'
#' Alternative way in RMD (but not implemented in this function) - for ValueTables:
#' "@VAL[labelCell:labelCol:valueCol](labelTable)"
#'
#' @param labelTable Character string, specifying the label of the table. Labels are limited to
#'   40 characters and should not include whitespaces.
#' @param row Numeric value (integer) defining the row of the element to report.
#'   Header row not counted.
#' @param col Numeric value (integer) defining the column of the element to report.
#' @param labelValue Character string with the element in the "label" column of a
#'   ValueTable. If used then row and col need to be NULL and vice versa.
#' @return Table element value cross reference command as IQRrmd object
#' @export
#' @family IQR report
rmdVALref <- function(labelTable,row=NULL,col=NULL,labelValue=NULL) {
  if (!is.null(labelValue)) {
    out__ <- paste0(" @VAL[",labelValue,"](",labelTable,") ")
  } else {
    out__ <- paste0(" @VAL[",row,",",col,"](",labelTable,") ")
  }
  class(out__) <- "IQRrmd"
  return (out__)
}



### !TEXTSTYLE element ----
###' Change the size, color, and/or style of a text piece
#'
#' @param text The text to which the changes should apply.
#' @param color string with color name ("blue", "black", "orange", "pink", etc.) In addition to
#'   color names also "#ffeedd" type of definition can be used.
#' @param size Numeric value (integer) defining the font size.
#' @param style Word character style to use. Note it has to be a Character style! Otherwise nothing is happening!
#' @return Textelement as IQRrmd object
#' @export
#' @family IQR report
rmdTEXTSTYLE <- function(text,color=NULL,size=NULL,style=NULL) {
  if (is.null(color) & is.null(size) & is.null(style)) {
    out__ <- text
  } else {
    out__ <- "!TEXTSTYLE["
    x1__ <- NULL
    x2__ <- NULL
    x3__ <- NULL
    if (!is.null(color)) x1__ <- paste0("color=",color)
    if (!is.null(size)) x2__ <- paste0("size=",size)
    if (!is.null(style)) x3__ <- paste0("style=",style)
    out__ <- paste0(out__,paste0(c(x1__,x2__,x3__),collapse = ","),"](",text,")")
  }
  class(out__) <- "IQRrmd"
  return (out__)
}

