###
# IQReport
###
#' Interface to the IQReport command line interface tool
#'
#' Allows to generate Word DOCX files from IQReport specific rmd files.
#' IQReport is not shipped with IQRtools but has to be obtained and installed
#' separately (http://www.intiquan.com/iqreport/).
#'
#' @param RMDfilePath Relative or absolute path to the RMD file.
#' @return No return value. But the generated DOCX file is available in the
#'         same folder as the RMD file.
#' @export
IQReport <- function(RMDfilePath){

  loadSetupOptions_IQReportAPI()

  # Check if IQReport is available
  if (!has_IQReport_executable())
    stop("The IQReport executable has not been installed.\nYou can obtain IQReport from here: http://www.intiquan.com/iqreport-the-reporting-solution/")

  # Check if RMDfilePath points to an rmd file
  RMDfilePath <- paste0(aux_strrep(RMDfilePath,".rmd",""),".rmd")
  if (!file.exists(RMDfilePath))
    stop("The provided RMDfilePath argument does not point to an '.rmd' file (case sensitive)")

  # Check if path correct
  if (grepl("~",RMDfilePath,fixed = TRUE))
    stop("The use of '~' in path names is not allowed. Please use a relative path instead!")
  if (grepl(":",RMDfilePath,fixed = TRUE))
    stop("The use of ':' in path names is not allowed. Please use a relative path instead!")


  if (.Platform$OS.type=="windows") {
    # Convert provided path to absolute path
    oldpath <- getwd()
    fp      <- aux_fileparts(RMDfilePath)
    setwd(fp$pathname)
    RMDfilePathAbsolute <- paste0(getwd(),"/",fp$filename,fp$fileext)
    setwd(oldpath)

    # Construct system call for IQReport
    callIQReport <- paste0(.PATH_IQRreport,' "',RMDfilePathAbsolute,'"')

    # Change into the IQReport install folder and execute IQReport and change back
    oldpath <- getwd()
    setwd(aux_fileparts(.PATH_IQRreport)$pathname)
    system(callIQReport)
    setwd(oldpath)
  } else {
    system(paste0("IQReport.sh ",RMDfilePath))
  }

  aux_rmdir("tmp")
  unlink("*.rmd.bak")
}

###
# has.IQReport
###
#' Check if IQReport is installed and available
#'
#' @return Returns TRUE if IQReport is available on the system
#' @export
has_IQReport_executable <- function(){
  # Load
  loadSetupOptions_IQReportAPI()
  # Check if PATH_IQReport is defined
  if (file.exists(.PATH_IQRreport)) return(TRUE)
  # Otherwise return false
  return(FALSE)
}


