% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nonmem_table_parsing.R
\name{tableNM_IQReportAPI}
\alias{tableNM_IQReportAPI}
\title{Imports NONMEM results and formats it as a parameter estimation table}
\usage{
tableNM_IQReportAPI(
  NMcontrolFile,
  NMxmlFile = NULL,
  FLAGtransform = TRUE,
  SIGNIF = 3,
  filename = NULL
)
}
\arguments{
\item{NMcontrolFile}{Path to the NONMEM control file}

\item{NMxmlFile}{Path to the NONMEM XML output file. If not provided it will
be assumes that the XML file is in the same folder as the control file and
that the difference in name is just the extension (.xml)}

\item{FLAGtransform}{MU referencing requires the use of transformed THETAs for
fixed effects of typical values. If this flag is set to TRUE, back transformation
is done to report the parameters in the domain of human interest. If FALSE is used
then the parameters are reported as estimated by NONMEM. Correct back transformation
requires providing the "Transformation" information in the model annotation.
"N" for untransformed (default), "L" for log transformed, and "G" for logit transformed.}

\item{SIGNIF}{Number of significant digits for the values, RSEs, and shrinkage. The
objective function, AIC, BIC will be rounded to the closest integer.}

\item{filename}{File name with path to where to store the exported table file.}
}
\value{
List with parsed information - self explaining
}
\description{
The estimation table can be exported to a table text file, ready for reporting
to MS Word with IQReport. Note that this is a standard format for the output table.
Other formats can easily be generated programatically or on request.
The user can optionally provide annotation information in the NONMEM control file
that will be used to annoate the imported results in a reasonable manner.
The format of that annotation is described below.
}
\details{
NONMEM model annotation is a piece of commented text in a NONMEM control file that
can appear at any place in the NONMEM control file but ideally appears at the start.
It is bounded by the key words "MODEL ANNOTATION START" and "MODEL ANNOTATION END" which
each are only allowed to appear once in the NONMEM control file and the start version should
come before the end version.\preformatted{; ==MODEL ANNOTATION START==
; Documentation of the annotation format:
; ---------------------------------------
;  - Only lines starting with "; THETAn", "; ETAn", and "; SIGMAn" are interpreted.
;    The other lines are simply ignored and can be used for further documentation
;    of the model - if desired.
;  - THETA, ETA, and SIGMA information can be provided in any order. Here in this
;    example we order them in blocks.
;  - Definition of THETAs happens as shown below with a comma separated list of
;    information. As the list is comma separated, commata in elements need to be
;    avoided. The entries are documented in the order of required appearance:
;      1) Name: Parameter name
;      2) COV:  1=parameter is a covariate coefficient,
;               0=parameter is not a covariate coefficient
;      3) ERR:  1=parameter is an error model parameter (if coded with THETAS),
;               0=parameter is not an error model parameter
;      4) Unit: Unit of the parameter
;      5) Description: Description of parameter
;      6) Transformation: N=untransformed, L=log transformed, G=logit transformed.
;                 Only used for parameters that are neither covariates nor error model
;                 parameters. Useful if mu-referencing is used as the back transformed
;                 parameters will be determined (including their standard errors and
;                 RSEs). Transformation information is optional and "N" will be assumed
;                 by default.
;
;          Name,        COV, ERR, Unit,     Description,                                 Transformation
; -----------------------------------------------------------------------------------------------------
; THETA1:  Ka,          0,   0,   1/hour,   First-order Absorption Rate Constant,        N
; THETA2:  F1Sty1107,   0,   0,   fraction, Absolute bioavailability for Study 1107,     L
; THETA3:  F1others,    0,   0,   fraction, Absolute bioavailability for All SC Studies, G
; THETA4:  CL,          0,   0,   L/hour,   Central Clearance,                           N
; THETA5:  Vc,          0,   0,   L,        Central Volume of Distribution,              N
; THETA6:  Q,           0,   0,   L/hour,   Distributional Clearance,                    N
; THETA7:  Vp,          0,   0,   L,        Peripheral Volume of Distribution,           N
; THETA8:  betaCLWT,    1,   0,   .,        Power for Weight on CL,
; THETA9:  betaVcWT,    1,   0,   .,        Power for Weight on Vc,
; THETA10: betaF1WT,    1,   0,   .,        Power for Weight on F1,
; THETA11: betaF1AGE,   1,   0,   .,        Power for Age on F1,
; THETA12: betaVpWT,    1,   0,   .,        Power for Weight on Vp,
;
; For ETAs only the parameter name and its description needs to be provided. Again comma
; separated information.
;
;          Name,        Description
; ---------------------------------------------------------------------------
; ETA1:    IIV(Vc),     IIV in Vc [exp]
; ETA2:    IIV(CL),     IIV in CL [exp]
; ETA3:    IIV(Q),      IIV in Q  [exp]
; ETA4:    IIV(Vp),     IIV in Vp [exp]
; ETA5:    IIV(Ka),     IIV in Ka [exp]
;
; The definition of the SIGMA information is not required if THETAs have been used
; to code error model parameters. Otherwise, if error model parameters have been coded
; using SIGMAs, then the annotation information can be provided here.
;
;          Name,        Unit,         Description
; ---------------------------------------------------------------------------
; SIGMA1:  ERR1,        log(ng/mL),   RV (log scale, IV Full-Profile) [add]
; SIGMA2:  ERR2,        log(ng/mL),   RV (log scale, IV Phase 2/3) [add]
; SIGMA3:  ERR3,        log(ng/mL),   RV (log scale, SC Full-Profile) [add]
; SIGMA4:  ERR4,        log(ng/mL),   RV (log scale, SC Phase 3) [add]
; ==MODEL ANNOTATION END==
}
}
\examples{
setwd("~")

# An example NONMEM model is provided in the package - in the same folder is also
# the model.xml NONMEM output in XML format that is used to load the results
NMcontrolFile <- system.file(package = "IQReportAPI","example/NONMEM/model.nmctl")

# Have a look at the annotation and the model
file.edit(NMcontrolFile)

# Generate a parameters table
table <- tableNM_IQReportAPI(NMcontrolFile)

# Look at the table
table

# Export table to file
tableNM_IQReportAPI(NMcontrolFile,filename="TAB01_NONMEM_Result.txt")

# Generate a simple Word file with this table
rmd <- rmdTABLE(file = "TAB01_NONMEM_Result.txt",fontsize = 9) + rmdNOINTRO()
export_IQRrmd(rmd,"exampleNM.rmd")
IQReport("exampleNM.rmd")

# Done ... that was easy - Enjoy!
}
